@@ -1,6 +1,8 @@
 /*******************************************************************************
  * Copyright (C) 2011, Mathias Kinzler <mathias.kinzler@sap.com>
  * Copyright (C) 2010, Jens Baumgart <jens.baumgart@sap.com>
+ * Copyright (C) 2012, Robin Stocker <robin@nibor.org>
+ * Copyright (C) 2012, François Rey <eclipse.org_@_francois_._rey_._name>
  *
  * All rights reserved. This program and the accompanying materials
  * are made available under the terms of the Eclipse Public License v1.0
@@ -12,12 +14,16 @@ package org.eclipse.egit.core.test;
 import java.io.File;
 import java.io.FileWriter;
 import java.io.IOException;
+import java.net.URISyntaxException;
 import java.util.Collection;
 import java.util.Collections;
 import java.util.regex.Pattern;
 
+import org.eclipse.core.filesystem.EFS;
 import org.eclipse.core.resources.IFile;
 import org.eclipse.core.resources.IProject;
+import org.eclipse.core.resources.IResource;
+import org.eclipse.core.resources.IResourceVisitor;
 import org.eclipse.core.runtime.CoreException;
 import org.eclipse.core.runtime.Path;
 import org.eclipse.egit.core.Activator;
@@ -44,6 +50,7 @@ import org.eclipse.jgit.lib.Repository;
 import org.eclipse.jgit.revwalk.RevCommit;
 import org.eclipse.jgit.revwalk.RevWalk;
 import org.eclipse.jgit.storage.file.FileRepository;
+import org.eclipse.jgit.transport.URIish;
 import org.eclipse.jgit.treewalk.TreeWalk;
 import org.eclipse.jgit.util.FileUtils;
 
@@ -241,6 +248,31 @@ public class TestRepository {
 				.toString());
 		new Git(repository).add().addFilepattern(repoPath).call();
 	}
+
+	/**
+	 * Adds all project files to version control
+	 *
+	 * @param project
+	 * @throws CoreException
+	 */
+	public void trackAllFiles(IProject project) throws CoreException {
+		project.accept(new IResourceVisitor() {
+
+			public boolean visit(IResource resource) throws CoreException {
+				if (resource instanceof IFile) {
+					try {
+						track(EFS.getStore(resource.getLocationURI())
+										.toLocalFile(0, null));
+					} catch (Exception e) {
+						throw new CoreException(Activator.error(e.getMessage(),
+								e));
+					}
+				}
+				return true;
+			}
+		});
+	}
+
 	/**
 	 * Removes file from version control
 	 *
@@ -321,16 +353,16 @@ public class TestRepository {
 
 
 	/**
-	 * Adds the given file to the index
+	 * Adds the given resource to the index
 	 *
-	 * @param file
+	 * @param resource
 	 * @throws CoreException
 	 * @throws IOException
 	 * @throws GitAPIException
 	 * @throws NoFilepatternException
 	 */
-	public void addToIndex(IFile file) throws CoreException, IOException, NoFilepatternException, GitAPIException {
-		String repoPath = getRepoRelativePath(file.getLocation().toOSString());
+	public void addToIndex(IResource resource) throws CoreException, IOException, NoFilepatternException, GitAPIException {
+		String repoPath = getRepoRelativePath(resource.getLocation().toOSString());
 		new Git(repository).add().addFilepattern(repoPath).call();
 	}
 
@@ -415,18 +447,18 @@ public class TestRepository {
 		}
 	}
 
-	public boolean inIndex(String path) throws IOException {
-		return getDirCacheEntry(path) != null;
+	public boolean inIndex(String absolutePath) throws IOException {
+		return getDirCacheEntry(absolutePath) != null;
 	}
 
-	public boolean removedFromIndex(String path) throws IOException {
-		DirCacheEntry dc = getDirCacheEntry(path);
+	public boolean removedFromIndex(String absolutePath) throws IOException {
+		DirCacheEntry dc = getDirCacheEntry(absolutePath);
 		if (dc == null)
 			return true;
 
 		Ref ref = repository.getRef(Constants.HEAD);
 		RevCommit c = new RevWalk(repository).parseCommit(ref.getObjectId());
-		TreeWalk tw = TreeWalk.forPath(repository, path, c.getTree());
+		TreeWalk tw = TreeWalk.forPath(repository, getRepoRelativePath(absolutePath), c.getTree());
 
 		return tw == null || dc.getObjectId().equals(tw.getObjectId(0));
 	}
@@ -468,8 +500,10 @@ public class TestRepository {
 	}
 
 	public void dispose() {
-		repository.close();
-		repository = null;
+		if (repository != null) {
+			repository.close();
+			repository = null;
+		}
 	}
 
 	/**
@@ -498,6 +532,10 @@ public class TestRepository {
 		disconnect.execute(null);
 	}
 
+	public URIish getUri() throws URISyntaxException {
+		return new URIish("file:///" + repository.getDirectory().toString());
+	}
+
 	private DirCacheEntry getDirCacheEntry(String path) throws IOException {
 		String repoPath = getRepoRelativePath(path);
 		DirCache dc = DirCache.read(repository.getIndexFile(), repository.getFS());
