@@ -30,6 +30,9 @@ import org.eclipse.core.runtime.IStatus;
 import org.eclipse.core.runtime.Path;
 import org.eclipse.core.runtime.Status;
 import org.eclipse.core.runtime.SubProgressMonitor;
+import org.eclipse.egit.core.internal.indexdiff.IndexDiffCache;
+import org.eclipse.egit.core.internal.indexdiff.IndexDiffCacheEntry;
+import org.eclipse.egit.core.internal.indexdiff.IndexDiffData;
 import org.eclipse.egit.core.project.GitProjectData;
 import org.eclipse.egit.core.project.RepositoryMapping;
 import org.eclipse.jgit.dircache.DirCache;
@@ -65,10 +68,27 @@ class GitMoveDeleteHook implements IMoveDeleteHook {
 		if (map == null)
 			return false;
 
+		String repoRelativePath = map.getRepoRelativePath(file);
+		IndexDiffCache indexDiffCache = Activator.getDefault()
+				.getIndexDiffCache();
+		IndexDiffCacheEntry indexDiffCacheEntry = indexDiffCache
+				.getIndexDiffCacheEntry(map.getRepository());
+		IndexDiffData indexDiff = indexDiffCacheEntry.getIndexDiff();
+		if (indexDiff != null) {
+			if (indexDiff.getUntracked().contains(repoRelativePath))
+				return false;
+			if (indexDiff.getIgnoredNotInIndex().contains(repoRelativePath))
+				return false;
+		}
+		if (!file.exists())
+			return false;
+		if (file.isDerived())
+			return false;
+
 		DirCache dirc = null;
 		try {
 			dirc = map.getRepository().lockDirCache();
-			final int first = dirc.findEntry(map.getRepoRelativePath(file));
+			final int first = dirc.findEntry(repoRelativePath);
 			if (first < 0) {
 				dirc.unlock();
 				return false;
@@ -131,9 +151,13 @@ class GitMoveDeleteHook implements IMoveDeleteHook {
 			sCache = srcm.getRepository().lockDirCache();
 			final String sPath = srcm.getRepoRelativePath(srcf);
 			final DirCacheEntry sEnt = sCache.getEntry(sPath);
-			if (sEnt == null) {
-				sCache.unlock();
-				return false;
+			if (sEnt == null)
+				return FINISH_FOR_ME;
+
+			if (!sEnt.isMerged()) {
+				tree.failed(new Status(IStatus.WARNING, Activator.getPluginId(),
+						CoreText.MoveDeleteHook_unmergedFileError));
+				return I_AM_DONE;
 			}
 
 			final DirCacheEditor sEdit = sCache.editor();
@@ -159,7 +183,7 @@ class GitMoveDeleteHook implements IMoveDeleteHook {
 			if (sCache != null)
 				sCache.unlock();
 		}
-		return true;
+		return I_AM_DONE;
 	}
 
 	public boolean moveFolder(final IResourceTree tree, final IFolder srcf,
@@ -190,6 +214,10 @@ class GitMoveDeleteHook implements IMoveDeleteHook {
 				case UNTRACKED:
 					// we are not responsible for moving untracked files
 					return FINISH_FOR_ME;
+				case UNMERGED:
+					tree.failed(new Status(IStatus.WARNING, Activator.getPluginId(),
+							CoreText.MoveDeleteHook_unmergedFileInFolderError));
+					return I_AM_DONE;
 				}
 			}
 			tree.standardMoveFolder(srcf, dstf, updateFlags, monitor);
@@ -312,6 +340,10 @@ class GitMoveDeleteHook implements IMoveDeleteHook {
 			case UNTRACKED:
 				// we are not responsible for moving untracked files
 				return FINISH_FOR_ME;
+			case UNMERGED:
+				tree.failed(new Status(IStatus.WARNING, Activator.getPluginId(),
+						CoreText.MoveDeleteHook_unmergedFileInFolderError));
+				return I_AM_DONE;
 			}
 
 			tree.standardMoveProject(source, description, updateFlags,
@@ -359,7 +391,7 @@ class GitMoveDeleteHook implements IMoveDeleteHook {
 		return true; // We're done with the move
 	}
 
-	enum MoveResult{SUCCESS, FAILED, UNTRACKED}
+	enum MoveResult { SUCCESS, FAILED, UNTRACKED, UNMERGED }
 
 	private MoveResult moveIndexContent(String dPath,
 			final RepositoryMapping srcm, final String sPath) throws IOException {
@@ -377,6 +409,8 @@ class GitMoveDeleteHook implements IMoveDeleteHook {
 			final int sPathLen = sPath.length() == 0 ? sPath.length() : sPath
 					.length() + 1;
 			for (final DirCacheEntry se : sEnt) {
+				if (!se.isMerged())
+					return MoveResult.UNMERGED;
 				final String p = se.getPathString().substring(sPathLen);
 				sEdit.add(new DirCacheEditor.PathEdit(dPath + p) {
 					@Override
