@@ -1,5 +1,5 @@
 /*******************************************************************************
- * Copyright (C) 2006, Robin Rosenberg <robin.rosenberg@dewire.com>
+ * Copyright (C) 2006, 2012 Robin Rosenberg <robin.rosenberg@dewire.com>
  * Copyright (C) 2008, Shawn O. Pearce <spearce@spearce.org>
  * Copyright (C) 2011, Dariusz Luksza <dariusz@luksza.org>
  *
@@ -26,6 +26,8 @@ import org.eclipse.jgit.errors.MissingObjectException;
 import org.eclipse.jgit.lib.Constants;
 import org.eclipse.jgit.lib.ObjectId;
 import org.eclipse.jgit.lib.Repository;
+import org.eclipse.jgit.treewalk.WorkingTreeOptions;
+import org.eclipse.jgit.util.io.AutoCRLFInputStream;
 import org.eclipse.osgi.util.NLS;
 
 /** Accesses a blob from Git. */
@@ -62,7 +64,17 @@ class BlobStorage implements IStorage {
 			return new ByteArrayInputStream(new byte[0]);
 
 		try {
-			return db.open(blobId, Constants.OBJ_BLOB).openStream();
+			WorkingTreeOptions workingTreeOptions = db.getConfig().get(WorkingTreeOptions.KEY);
+			switch (workingTreeOptions.getAutoCRLF()) {
+			case INPUT:
+				// When autocrlf == input the working tree could be either CRLF or LF, i.e. the comparison
+				// itself should ignore line endings.
+			case FALSE:
+				return db.open(blobId, Constants.OBJ_BLOB).openStream();
+			case TRUE:
+			default:
+				return new AutoCRLFInputStream(db.open(blobId, Constants.OBJ_BLOB).openStream(), true);
+			}
 		} catch (MissingObjectException notFound) {
 			throw new CoreException(Activator.error(NLS.bind(
 					CoreText.BlobStorage_blobNotFound, blobId.name(), path),
