@@ -3,6 +3,7 @@
  * Copyright (C) 2006, Robin Rosenberg <robin.rosenberg@dewire.com>
  * Copyright (C) 2008, Shawn O. Pearce <spearce@spearce.org>
  * Copyright (C) 2011, Dariusz Luksza <dariusz@luksza.org>
+ * Copyright (C) 2013, Robin Stocker <robin@nibor.org>
  *
  * All rights reserved. This program and the accompanying materials
  * are made available under the terms of the Eclipse Public License v1.0
@@ -18,6 +19,7 @@ import org.eclipse.core.runtime.CoreException;
 import org.eclipse.core.runtime.IProgressMonitor;
 import org.eclipse.egit.core.Activator;
 import org.eclipse.egit.core.CoreText;
+import org.eclipse.jgit.dircache.DirCache;
 import org.eclipse.jgit.dircache.DirCacheEntry;
 import org.eclipse.jgit.lib.ObjectId;
 import org.eclipse.jgit.lib.Repository;
@@ -25,17 +27,28 @@ import org.eclipse.osgi.util.NLS;
 import org.eclipse.team.core.history.IFileRevision;
 
 /** An {@link IFileRevision} for the version in the Git index. */
-class IndexFileRevision extends GitFileRevision implements IFileRevision {
+class IndexFileRevision extends GitFileRevision {
+
+	// This is to maintain compatibility with the old behavior
+	private static final int FIRST_AVAILABLE = -1;
+
 	private final Repository db;
 
 	private final String path;
 
+	private final int stage;
+
 	private ObjectId blobId;
 
 	IndexFileRevision(final Repository repo, final String fileName) {
+		this(repo, fileName, FIRST_AVAILABLE);
+	}
+
+	IndexFileRevision(final Repository repo, final String fileName, int stage) {
 		super(fileName);
-		db = repo;
-		path = fileName;
+		this.db = repo;
+		this.path = fileName;
+		this.stage = stage;
 	}
 
 	public IStorage getStorage(IProgressMonitor monitor) throws CoreException {
@@ -48,11 +61,6 @@ class IndexFileRevision extends GitFileRevision implements IFileRevision {
 		return false;
 	}
 
-	public IFileRevision withAllProperties(IProgressMonitor monitor)
-			throws CoreException {
-		return null;
-	}
-
 	public String getAuthor() {
 		return "";  //$NON-NLS-1$
 	}
@@ -71,14 +79,27 @@ class IndexFileRevision extends GitFileRevision implements IFileRevision {
 
 	private ObjectId locateBlobObjectId() throws CoreException {
 		try {
-			DirCacheEntry entry = db.readDirCache().getEntry(path);
-			if (entry == null)
+			DirCache dc = db.readDirCache();
+			int firstIndex = dc.findEntry(path);
+			if (firstIndex < 0)
 				return null;
 
-			return entry.getObjectId();
+			// Try to avoid call to nextEntry if first entry already matches
+			DirCacheEntry firstEntry = dc.getEntry(firstIndex);
+			if (stage == FIRST_AVAILABLE || firstEntry.getStage() == stage)
+				return firstEntry.getObjectId();
+
+			// Ok, we have to search
+			int nextIndex = dc.nextEntry(firstIndex);
+			for (int i = firstIndex; i < nextIndex; i++) {
+				DirCacheEntry entry = dc.getEntry(i);
+				if (entry.getStage() == stage)
+					return entry.getObjectId();
+			}
+			return null;
 		} catch (IOException e) {
 			throw new CoreException(Activator.error(NLS.bind(
 					CoreText.IndexFileRevision_errorLookingUpPath, path), e));
 		}
 	}
-}
\ No newline at end of file
+}
