@@ -3,6 +3,7 @@
  * Copyright (C) 2007, Martin Oberhuber (martin.oberhuber@windriver.com)
  * Copyright (C) 2008, Robin Rosenberg <robin.rosenberg@dewire.com>
  * Copyright (C) 2010, Jens Baumgart <jens.baumgart@sap.com>
+ * Copyright (C) 2012, 2013 Robin Stocker <robin@nibor.org>
  *
  * All rights reserved. This program and the accompanying materials
  * are made available under the terms of the Eclipse Public License v1.0
@@ -14,14 +15,19 @@ package org.eclipse.egit.core.internal.util;
 import java.io.File;
 import java.io.IOException;
 import java.util.ArrayList;
+import java.util.Arrays;
 import java.util.Collection;
+import java.util.Comparator;
 import java.util.HashSet;
+import java.util.LinkedHashSet;
 import java.util.List;
 import java.util.Set;
 
+import org.eclipse.core.resources.IContainer;
 import org.eclipse.core.resources.IProject;
 import org.eclipse.core.resources.IProjectDescription;
 import org.eclipse.core.resources.IResource;
+import org.eclipse.core.resources.IWorkspaceRoot;
 import org.eclipse.core.resources.ResourcesPlugin;
 import org.eclipse.core.runtime.CoreException;
 import org.eclipse.core.runtime.IPath;
@@ -249,19 +255,111 @@ public class ProjectUtil {
 		return result.toArray(new IProject[result.size()]);
 	}
 
+	/**
+	 * The method returns all projects containing at least one of the given
+	 * paths.
+	 *
+	 * @param repository
+	 *            the repository who's working tree is used as base for lookup
+	 * @param fileList
+	 *            the list of files/directories to lookup
+	 * @return valid projects containing one of the paths
+	 * @throws CoreException
+	 */
+	public static IProject[] getProjectsContaining(Repository repository,
+			Collection<String> fileList) throws CoreException {
+		Set<IProject> result = new LinkedHashSet<IProject>();
+		File workTree = repository.getWorkTree();
+
+		IWorkspaceRoot root = ResourcesPlugin.getWorkspace().getRoot();
+		IProject[] projects = getProjectsForContainerMatch(root);
+
+		for (String member : fileList) {
+			File file = new File(workTree, member);
+
+			for (IProject prj : projects) {
+				if (checkContainerMatch(prj, file.getAbsolutePath())) {
+					result.add(prj);
+					break;
+				}
+			}
+		}
+
+		return result.toArray(new IProject[result.size()]);
+	}
+
+	/**
+	 * Looks up the IProject containing the given file, if available. This is
+	 * done by path comparison, which is very cheap compared to
+	 * IWorkspaceRoot.findContainersForLocationURI(). If no project is found the
+	 * code returns the {@link IWorkspaceRoot} or the file is inside the
+	 * workspace.
+	 *
+	 * @param file
+	 *            the path to lookup a container for
+	 * @return the IProject or IWorkspaceRoot or <code>null</code> if not found.
+	 */
+	public static IContainer findProjectOrWorkspaceRoot(File file) {
+		String absFile = file.getAbsolutePath();
+		IWorkspaceRoot root = ResourcesPlugin.getWorkspace().getRoot();
+		IProject[] allProjects = getProjectsForContainerMatch(root);
+
+		for (IProject prj : allProjects)
+			if (checkContainerMatch(prj, absFile))
+				return prj;
+
+		if (checkContainerMatch(root, absFile))
+			return root;
+
+		return null;
+	}
+
+	private static IProject[] getProjectsForContainerMatch(IWorkspaceRoot root) {
+		IProject[] allProjects = root.getProjects();
+
+		// Sorting makes us look into nested projects first
+		Arrays.sort(allProjects, new Comparator<IProject>() {
+			public int compare(IProject o1, IProject o2) {
+				return -o1.getLocation().toFile()
+						.compareTo(o2.getLocation().toFile());
+			}
+
+		});
+		return allProjects;
+	}
+
+	private static boolean checkContainerMatch(IContainer container,
+			String absFile) {
+		String absPrj = container.getLocation().toFile().getAbsolutePath();
+		if (absPrj.equals(absFile))
+			return true;
+		if (absPrj.length() < absFile.length()) {
+			char sepChar = absFile.charAt(absPrj.length());
+			if (sepChar == File.separatorChar && absFile.startsWith(absPrj))
+				return true;
+		}
+		return false;
+	}
+
 	/**
 	 * Find directories containing .project files recursively starting at given
 	 * directory
 	 *
-	 * @param files
-	 * @param directory
-	 * @param visistedDirs
+	 * @param files the collection to add the found projects to
+	 * @param directory where to search for project files
+	 * @param searchNested whether to search for nested projects or not
 	 * @param monitor
 	 * @return true if projects files found, false otherwise
 	 */
 	public static boolean findProjectFiles(final Collection<File> files,
-			final File directory, final Set<String> visistedDirs,
+			final File directory, boolean searchNested,
 			final IProgressMonitor monitor) {
+		return findProjectFiles(files, directory, searchNested, null, monitor);
+	}
+
+	private static boolean findProjectFiles(final Collection<File> files,
+			final File directory, final boolean searchNested,
+			final Set<String> visistedDirs, final IProgressMonitor monitor) {
 		if (directory == null)
 			return false;
 
@@ -295,17 +393,18 @@ public class ProjectUtil {
 			directoriesVisited = visistedDirs;
 
 		// first look for project description files
+		boolean foundProject = false;
 		final String dotProject = IProjectDescription.DESCRIPTION_FILE_NAME;
 		for (int i = 0; i < contents.length; i++) {
 			File file = contents[i];
 			if (file.isFile() && file.getName().equals(dotProject)) {
 				files.add(file);
-				// don't search sub-directories since we can't have nested
-				// projects
-				return true;
+				foundProject = true;
 			}
 		}
-		// no project description found, so recurse into sub-directories
+		if (foundProject && !searchNested)
+			return true;
+		// recurse into sub-directories (even when project was found above, for nested projects)
 		for (int i = 0; i < contents.length; i++) {
 			// Skip non-directories
 			if (!contents[i].isDirectory())
@@ -322,7 +421,8 @@ public class ProjectUtil {
 				Activator.logError(exception.getLocalizedMessage(), exception);
 
 			}
-			findProjectFiles(files, contents[i], directoriesVisited, pm);
+			findProjectFiles(files, contents[i], searchNested,
+					directoriesVisited, pm);
 		}
 		return true;
 	}
