@@ -105,9 +105,8 @@ public class BranchOperation extends BaseOperation {
 			public void run(IProgressMonitor pm) throws CoreException {
 				preExecute(pm);
 
-				IProject[] validProjects = ProjectUtil
-						.getValidOpenProjects(repository);
-				IProject[] missing = getMissingProjects(target, validProjects);
+				IProject[] missing = getMissingProjects(target, ProjectUtil
+						.getValidOpenProjects(repository));
 
 				pm.beginTask(NLS.bind(
 						CoreText.BranchOperation_performingBranch, target),
@@ -143,7 +142,14 @@ public class BranchOperation extends BaseOperation {
 				if (result.getStatus() == Status.NONDELETED)
 					retryDelete(result.getUndeletedList());
 				pm.worked(1);
-				ProjectUtil.refreshValidProjects(validProjects, delete,
+
+				List<String> pathsToHandle = new ArrayList<String>();
+				pathsToHandle.addAll(co.getResult().getModifiedList());
+				pathsToHandle.addAll(co.getResult().getRemovedList());
+				pathsToHandle.addAll(co.getResult().getConflictList());
+				IProject[] refreshProjects = ProjectUtil
+						.getProjectsContaining(repository, pathsToHandle);
+				ProjectUtil.refreshValidProjects(refreshProjects, delete,
 						new SubProgressMonitor(pm, 1));
 				pm.worked(1);
 
