@@ -1,6 +1,7 @@
 /*******************************************************************************
  * Copyright (C) 2008, Marek Zawirski <marek.zawirski@gmail.com>
  * Copyright (C) 2011, Mathias Kinzler <mathias.kinzler@sap.com>
+ * Copyright (C) 2012, Robin Stocker <robin@nibor.org>
  *
  * All rights reserved. This program and the accompanying materials
  * are made available under the terms of the Eclipse Public License v1.0
@@ -11,8 +12,7 @@ package org.eclipse.egit.core.op;
 
 import java.lang.reflect.InvocationTargetException;
 import java.net.URISyntaxException;
-import java.util.ArrayList;
-import java.util.List;
+import java.util.Collection;
 
 import org.eclipse.core.runtime.IProgressMonitor;
 import org.eclipse.core.runtime.NullProgressMonitor;
@@ -25,10 +25,10 @@ import org.eclipse.jgit.api.errors.JGitInternalException;
 import org.eclipse.jgit.lib.Repository;
 import org.eclipse.jgit.transport.CredentialsProvider;
 import org.eclipse.jgit.transport.PushResult;
-import org.eclipse.jgit.transport.RefSpec;
 import org.eclipse.jgit.transport.RemoteConfig;
 import org.eclipse.jgit.transport.RemoteRefUpdate;
 import org.eclipse.jgit.transport.RemoteRefUpdate.Status;
+import org.eclipse.jgit.transport.Transport;
 import org.eclipse.jgit.transport.URIish;
 import org.eclipse.osgi.util.NLS;
 
@@ -166,37 +166,26 @@ public class PushOperation {
 						SubProgressMonitor.PREPEND_MAIN_LABEL_TO_SUBTASK);
 
 				try {
-					List<RefSpec> specs = new ArrayList<RefSpec>(3);
-					for (RemoteRefUpdate update : specification
-							.getRefUpdates(uri)) {
-						RefSpec spec = new RefSpec();
-						spec = spec.setSourceDestination(update.getSrcRef(),
-								update.getRemoteName());
-						spec = spec.setForceUpdate(update.isForceUpdate());
-						specs.add(spec);
-					}
 					if (monitor.isCanceled()) {
 						operationResult.addOperationResult(uri,
 								CoreText.PushOperation_resultCancelled);
 						continue;
 					}
 
+					Collection<RemoteRefUpdate> refUpdates = specification.getRefUpdates(uri);
 					final EclipseGitProgressTransformer gitSubMonitor = new EclipseGitProgressTransformer(
 							subMonitor);
 
 					try {
-						Iterable<PushResult> results = git.push().setRemote(
-								uri.toPrivateString()).setRefSpecs(specs)
-								.setDryRun(dryRun).setTimeout(timeout)
-								.setProgressMonitor(gitSubMonitor)
-								.setCredentialsProvider(credentialsProvider)
-								.call();
-						for (PushResult result : results) {
-							operationResult.addOperationResult(result.getURI(),
-									result);
-							specification.addURIRefUpdates(result.getURI(),
-									result.getRemoteUpdates());
-						}
+						Transport transport = Transport.open(localDb, uri);
+						transport.setDryRun(dryRun);
+						transport.setTimeout(timeout);
+						if (credentialsProvider != null)
+							transport.setCredentialsProvider(credentialsProvider);
+						PushResult result = transport.push(gitSubMonitor, refUpdates);
+
+						operationResult.addOperationResult(result.getURI(), result);
+						specification.addURIRefUpdates(result.getURI(), result.getRemoteUpdates());
 					} catch (JGitInternalException e) {
 						String errorMessage = e.getCause() != null ? e
 								.getCause().getMessage() : e.getMessage();
