@@ -4,6 +4,8 @@
  * Copyright (C) 2008, Shawn O. Pearce <spearce@spearce.org>
  * Copyright (C) 2008, Shunichi Fuji <palglowr@gmail.com>
  * Copyright (C) 2008, Google Inc.
+ * Copyright (C) 2012, 2013 Robin Stocker <robin@nibor.org>
+ * Copyright (C) 2013, François Rey <eclipse.org_@_francois_._rey_._name>
  *
  * All rights reserved. This program and the accompanying materials
  * are made available under the terms of the Eclipse Public License v1.0
@@ -203,13 +205,15 @@ public class RepositoryMapping {
 	 * bother.
 	 *
 	 * @param rsrc
-	 * @return the path relative to the Git repository, including base name.
-	 *         <code>null</code> if the path cannot be determined.
+	 * @return the path relative to the Git repository, including base name. An
+	 *         empty string (<code>""</code>) if passed resource corresponds to
+	 *         working directory (root). <code>null</code> if the path cannot be
+	 *         determined.
 	 */
 	public String getRepoRelativePath(final IResource rsrc) {
 		IPath location = rsrc.getLocation();
 		if (location == null)
-			location = rsrc.getFullPath();
+			return null;
 		return getRepoRelativePath(location);
 	}
 
@@ -220,8 +224,10 @@ public class RepositoryMapping {
 	 * bother.
 	 *
 	 * @param location
-	 * @return the path relative to the Git repository, including base name.
-	 *         <code>null</code> if the path cannot be determined.
+	 * @return the path relative to the Git repository, including base name. An
+	 *         empty string (<code>""</code>) if passed location corresponds to
+	 *         working directory (root). <code>null</code> if the path cannot be
+	 *         determined.
 	 */
 	public String getRepoRelativePath(IPath location) {
 		final int pfxLen = workdirPrefix.length();
@@ -235,15 +241,20 @@ public class RepositoryMapping {
 	}
 
 	/**
-	 * Get the repository mapping for a resource
+	 * Get the repository mapping for a resource. If the given resource is a
+	 * linked resource, the raw location of the resource will be used to
+	 * determine a repository mapping.
 	 *
 	 * @param resource
-	 * @return the RepositoryMapping for this resource,
-	 *         or null for non GitProvider.
+	 * @return the RepositoryMapping for this resource, or null for non
+	 *         GitProvider.
 	 */
 	public static RepositoryMapping getMapping(final IResource resource) {
 		if (isNonWorkspace(resource))
-			return getMappingForNonWorkspaceResource(resource);
+			return null;
+
+		if (resource.isLinked(IResource.CHECK_ANCESTORS))
+			return getMapping(resource.getLocation());
 
 		IProject project = resource.getProject();
 		if (project == null)
@@ -267,7 +278,6 @@ public class RepositoryMapping {
 	 *         or null for non GitProvider.
 	 */
 	public static RepositoryMapping getMapping(IPath path) {
-		IPath fullPath = path.removeLastSegments(1);
 		IProject[] projects = ResourcesPlugin.getWorkspace().getRoot()
 				.getProjects();
 
@@ -279,7 +289,7 @@ public class RepositoryMapping {
 				continue;
 
 			Path workingTree = new Path(mapping.getWorkTree().toString());
-			IPath relative = fullPath.makeRelativeTo(workingTree);
+			IPath relative = path.makeRelativeTo(workingTree);
 			String firstSegment = relative.segment(0);
 
 			if (firstSegment == null || !"..".equals(firstSegment)) //$NON-NLS-1$
@@ -327,9 +337,4 @@ public class RepositoryMapping {
 		}
 		return gitDirAbsolutePath;
 	}
-
-	private static RepositoryMapping getMappingForNonWorkspaceResource(
-			final IResource resource) {
-		return getMapping(resource.getFullPath());
-	}
 }
