@@ -1,5 +1,5 @@
 /*******************************************************************************
- * Copyright (c) 2010 SAP AG.
+ * Copyright (c) 2010, 2012 SAP AG and others.
  * All rights reserved. This program and the accompanying materials
  * are made available under the terms of the Eclipse Public License v1.0
  * which accompanies this distribution, and is available at
@@ -24,7 +24,8 @@ import org.junit.runners.Suite.SuiteClasses;
 		ShowInTest.class, //
 		CompareActionsTest.class, //
 		ReplaceActionsTest.class, //
-		CreatePatchActionTest.class })
+		CreatePatchActionTest.class, //
+		MergeToolTest.class})
 public class AllTeamActionTests {
 	// nothing
 }
