@@ -1,5 +1,5 @@
 /*******************************************************************************
- * Copyright (c) 2010 SAP AG.
+ * Copyright (c) 2010, 2012 SAP AG and others.
  * All rights reserved. This program and the accompanying materials
  * are made available under the terms of the Eclipse Public License v1.0
  * which accompanies this distribution, and is available at
@@ -124,19 +124,19 @@ public class GitRepositoriesViewBranchHandlingTest extends
 		localItem.getNode(0).select();
 		assertCheckoutNotAvailable(view);
 		localItem.getNode(1).select();
-		ContextMenuHelper.clickContextMenu(view.bot().tree(), myUtil
+		ContextMenuHelper.clickContextMenuSync(view.bot().tree(), myUtil
 				.getPluginLocalizedValue("CheckoutCommand"));
 		TestUtil.joinJobs(JobFamilies.CHECKOUT);
 
 		assertCheckoutNotAvailable(view);
 
 		localItem.getNode(0).select();
-		ContextMenuHelper.clickContextMenu(view.bot().tree(), myUtil
+		ContextMenuHelper.clickContextMenuSync(view.bot().tree(), myUtil
 				.getPluginLocalizedValue("CheckoutCommand"));
 		TestUtil.joinJobs(JobFamilies.CHECKOUT);
 		localItem.getNode(1).select();
 		refreshAndWait();
-		ContextMenuHelper.clickContextMenu(bot.tree(), myUtil
+		ContextMenuHelper.clickContextMenuSync(view.bot().tree(), myUtil
 				.getPluginLocalizedValue("RepoViewDeleteBranch.label"));
 		refreshAndWait();
 		localItem = myRepoViewUtil.getLocalBranchesItem(view.bot().tree(),
@@ -196,7 +196,7 @@ public class GitRepositoriesViewBranchHandlingTest extends
 		TestUtil.joinJobs(JobFamilies.CHECKOUT);
 		localItem.getNode(1).select();
 		refreshAndWait();
-		ContextMenuHelper.clickContextMenu(bot.tree(), myUtil
+		ContextMenuHelper.clickContextMenu(view.bot().tree(), myUtil
 				.getPluginLocalizedValue("RepoViewDeleteBranch.label"));
 		SWTBotShell confirmPopup = bot
 				.shell(UIText.UnmergedBranchDialog_Title);
@@ -275,7 +275,7 @@ public class GitRepositoriesViewBranchHandlingTest extends
 			assertEquals("Wrong number of remote children", 2, children.size());
 
 			item.getNode("origin/stable").select();
-			ContextMenuHelper.clickContextMenu(tree, myUtil
+			ContextMenuHelper.clickContextMenuSync(tree, myUtil
 					.getPluginLocalizedValue("CheckoutCommand"));
 			TestUtil.joinJobs(JobFamilies.CHECKOUT);
 			refreshAndWait();
@@ -407,9 +407,9 @@ public class GitRepositoriesViewBranchHandlingTest extends
 				.bot().tree(), clonedRepositoryFile);
 		localItem.expand().getNode("configTest").select();
 
-		ContextMenuHelper.clickContextMenu(view.bot().tree(),
+		ContextMenuHelper.clickContextMenuSync(view.bot().tree(),
 				myUtil.getPluginLocalizedValue("ShowIn"),
-				myUtil.getPluginLocalizedValue("RepoViewOpenProperties.label"));
+				"Properties");
 
 		SWTBotView propsView = bot.viewByTitle("Properties");
 		SWTBotTreeItem rootItem = propsView
@@ -493,7 +493,7 @@ public class GitRepositoriesViewBranchHandlingTest extends
 
 		ContextMenuHelper.clickContextMenu(view.bot().tree(),
 				myUtil.getPluginLocalizedValue("ShowIn"),
-				myUtil.getPluginLocalizedValue("RepoViewOpenProperties.label"));
+				"Properties");
 
 		propsView = bot.viewByTitle("Properties");
 		rootItem = propsView
