@@ -1,5 +1,5 @@
 /*******************************************************************************
- * Copyright (C) 2010,2011 Dariusz Luksza <dariusz@luksza.org>
+ * Copyright (C) 2010-2013 Dariusz Luksza <dariusz@luksza.org> and others.
  *
  * All rights reserved. This program and the accompanying materials
  * are made available under the terms of the Eclipse Public License v1.0
@@ -12,7 +12,6 @@ import static org.eclipse.egit.ui.UIText.CommitAction_commit;
 import static org.eclipse.egit.ui.UIText.CommitDialog_Commit;
 import static org.eclipse.egit.ui.UIText.CommitDialog_CommitChanges;
 import static org.eclipse.egit.ui.UIText.CommitDialog_SelectAll;
-import static org.eclipse.egit.ui.UIText.GitModelWorkingTree_workingTree;
 import static org.eclipse.egit.ui.test.ContextMenuHelper.clickContextMenu;
 import static org.eclipse.egit.ui.test.TestUtil.waitUntilTreeHasNodeContainsText;
 import static org.eclipse.jface.dialogs.MessageDialogWithToggle.NEVER;
@@ -25,6 +24,8 @@ import java.io.IOException;
 import java.io.UnsupportedEncodingException;
 import java.util.ArrayList;
 import java.util.Arrays;
+import java.util.List;
+import java.util.concurrent.TimeUnit;
 
 import org.eclipse.core.resources.IFile;
 import org.eclipse.core.resources.IFolder;
@@ -33,12 +34,9 @@ import org.eclipse.core.resources.IProjectDescription;
 import org.eclipse.core.resources.IResource;
 import org.eclipse.core.resources.ResourcesPlugin;
 import org.eclipse.core.runtime.CoreException;
-import org.eclipse.core.runtime.NullProgressMonitor;
 import org.eclipse.core.runtime.Path;
-import org.eclipse.core.runtime.jobs.Job;
 import org.eclipse.egit.core.op.CommitOperation;
 import org.eclipse.egit.core.op.ConnectProviderOperation;
-import org.eclipse.egit.core.op.ResetOperation;
 import org.eclipse.egit.core.project.RepositoryMapping;
 import org.eclipse.egit.core.synchronize.dto.GitSynchronizeData;
 import org.eclipse.egit.ui.Activator;
@@ -46,31 +44,32 @@ import org.eclipse.egit.ui.JobFamilies;
 import org.eclipse.egit.ui.UIPreferences;
 import org.eclipse.egit.ui.common.LocalRepositoryTestCase;
 import org.eclipse.egit.ui.internal.synchronize.GitModelSynchronize;
-import org.eclipse.egit.ui.test.Eclipse;
+import org.eclipse.egit.ui.test.JobJoiner;
 import org.eclipse.egit.ui.test.TestUtil;
-import org.eclipse.jface.bindings.keys.KeyStroke;
 import org.eclipse.jgit.api.Git;
-import org.eclipse.jgit.api.ResetCommand.ResetType;
 import org.eclipse.jgit.lib.Constants;
 import org.eclipse.jgit.lib.Repository;
 import org.eclipse.jgit.storage.file.FileRepository;
+import org.eclipse.jgit.util.FileUtils;
+import org.eclipse.jgit.util.StringUtils;
 import org.eclipse.swtbot.eclipse.finder.widgets.SWTBotEditor;
 import org.eclipse.swtbot.eclipse.finder.widgets.SWTBotView;
 import org.eclipse.swtbot.swt.finder.SWTBot;
 import org.eclipse.swtbot.swt.finder.exceptions.WidgetNotFoundException;
 import org.eclipse.swtbot.swt.finder.waits.Conditions;
 import org.eclipse.swtbot.swt.finder.widgets.SWTBotStyledText;
-import org.eclipse.swtbot.swt.finder.widgets.SWTBotToolbarButton;
-import org.eclipse.swtbot.swt.finder.widgets.SWTBotToolbarDropDownButton;
 import org.eclipse.swtbot.swt.finder.widgets.SWTBotTree;
 import org.eclipse.swtbot.swt.finder.widgets.SWTBotTreeItem;
 import org.eclipse.team.internal.ui.TeamUIPlugin;
+import org.eclipse.team.ui.TeamUI;
+import org.eclipse.team.ui.mapping.ITeamContentProviderDescriptor;
+import org.eclipse.team.ui.mapping.ITeamContentProviderManager;
 import org.eclipse.team.ui.synchronize.ISynchronizeManager;
 import org.junit.After;
-import org.junit.AfterClass;
 import org.junit.Before;
 import org.junit.BeforeClass;
 
+@SuppressWarnings("restriction")
 public abstract class AbstractSynchronizeViewTest extends
 		LocalRepositoryTestCase {
 
@@ -82,7 +81,9 @@ public abstract class AbstractSynchronizeViewTest extends
 
 	protected static final String EMPTY_REPOSITORY = "EmptyRepository";
 
-	static File repositoryFile;
+	protected File repositoryFile;
+
+	protected File childRepositoryFile;
 
 	@Before public void setupViews() {
 		bot.perspectiveById("org.eclipse.jdt.ui.JavaPerspective").activate();
@@ -95,6 +96,27 @@ public abstract class AbstractSynchronizeViewTest extends
 		syncView.close();
 	}
 
+	@Before
+	public void setupRepository() throws Exception {
+		repositoryFile = createProjectAndCommitToRepository();
+		createAndCommitDotGitignore();
+
+		childRepositoryFile = createChildRepository(repositoryFile);
+
+		createTag(INITIAL_TAG);
+
+		Activator.getDefault().getRepositoryUtil()
+				.addConfiguredRepository(repositoryFile);
+	}
+
+	@After
+	public void deleteRepository() throws Exception {
+		deleteAllProjects();
+		shutDownRepositories();
+		FileUtils.delete(repositoryFile.getParentFile(), FileUtils.RECURSIVE | FileUtils.RETRY);
+		FileUtils.delete(childRepositoryFile.getParentFile(), FileUtils.RECURSIVE | FileUtils.RETRY);
+	}
+
 	@BeforeClass public static void setupEnvironment() throws Exception {
 		// disable perspective synchronize selection
 		TeamUIPlugin.getPlugin().getPreferenceStore().setValue(
@@ -102,21 +124,8 @@ public abstract class AbstractSynchronizeViewTest extends
 		Activator.getDefault().getPreferenceStore()
 				.setValue(UIPreferences.SYNC_VIEW_FETCH_BEFORE_LAUNCH, false);
 
-		repositoryFile = createProjectAndCommitToRepository();
-		createAndCommitDotGitignore();
-
-		createChildRepository(repositoryFile);
-		Activator.getDefault().getRepositoryUtil()
-				.addConfiguredRepository(repositoryFile);
-
 		bot.perspectiveById("org.eclipse.jdt.ui.JavaPerspective").activate();
 		bot.viewByTitle("Package Explorer").show();
-
-		createTag(INITIAL_TAG);
-	}
-
-	@AfterClass public static void restoreEnvironmentSetup() throws Exception {
-		new Eclipse().reset();
 	}
 
 	protected void changeFilesInProject() throws Exception {
@@ -138,20 +147,7 @@ public abstract class AbstractSynchronizeViewTest extends
 		coreTreeItem.collapse();
 	}
 
-	protected void resetRepositoryToCreateInitialTag() throws Exception {
-		Thread.sleep(2000);
-		ResetOperation rop = new ResetOperation(
-				lookupRepository(repositoryFile), Constants.R_TAGS +
-						INITIAL_TAG, ResetType.HARD);
-		rop.execute(new NullProgressMonitor());
-		Thread.sleep(2000);
-		rop = new ResetOperation(
-				lookupRepository(repositoryFile), Constants.R_TAGS +
-						INITIAL_TAG, ResetType.HARD);
-		rop.execute(new NullProgressMonitor());
-	}
-
-	public static void createTag(String tagName)
+	protected void createTag(String tagName)
 			throws Exception {
 		new Git(lookupRepository(repositoryFile)).tag().setName(tagName)
 				.setMessage(tagName).call();
@@ -169,13 +165,12 @@ public abstract class AbstractSynchronizeViewTest extends
 	}
 
 	protected void launchSynchronization(String srcRef, String dstRef,
-			boolean includeLocal) throws InterruptedException, IOException {
+			boolean includeLocal) throws IOException {
 		launchSynchronization(PROJ1, srcRef, dstRef, includeLocal);
 	}
 
 	protected void launchSynchronization(String projectName, String srcRef,
-			String dstRef, boolean includeLocal) throws InterruptedException,
-			IOException {
+			String dstRef, boolean includeLocal) throws IOException {
 		IProject project = ResourcesPlugin.getWorkspace().getRoot()
 				.getProject(projectName);
 		Repository repo = RepositoryMapping.getMapping(project).getRepository();
@@ -183,42 +178,17 @@ public abstract class AbstractSynchronizeViewTest extends
 		GitSynchronizeData data = new GitSynchronizeData(repo, srcRef, dstRef,
 				includeLocal);
 
+		JobJoiner jobJoiner = JobJoiner.startListening(
+				ISynchronizeManager.FAMILY_SYNCHRONIZE_OPERATION, 60,
+				TimeUnit.SECONDS);
 		GitModelSynchronize.launch(data, new IResource[] { project });
-
-		Job.getJobManager().join(JobFamilies.SYNCHRONIZE_READ_DATA, null);
-		Job.getJobManager().join(
-				ISynchronizeManager.FAMILY_SYNCHRONIZE_OPERATION, null);
+		jobJoiner.join();
 	}
 
-	protected void setGitChangeSetPresentationModel() throws Exception {
-		String modelName = util.getPluginLocalizedValue("ChangeSetModel.name");
-		setPresentationModel(modelName, "Show " + modelName);
-	}
-
-	protected SWTBot setPresentationModel(String model) throws Exception {
-		SWTBotView syncView = bot.viewByTitle("Synchronize");
-		SWTBotToolbarDropDownButton dropDown = syncView
-				.toolbarDropDownButton("Show File System Resources");
-		dropDown.menuItem(model).click();
-		// hide drop down
-		dropDown.pressShortcut(KeyStroke.getInstance("ESC"));
-
-		return syncView.bot();
-	}
-
-	protected SWTBot setPresentationModel(String modelName,
-			String toolbarDropDownTooltip) throws Exception {
-		SWTBotView syncView = bot.viewByTitle("Synchronize");
-		for (SWTBotToolbarButton button : syncView.getToolbarButtons())
-			if (button.getToolTipText().equals(toolbarDropDownTooltip)) {
-				SWTBotToolbarDropDownButton dropDown = (SWTBotToolbarDropDownButton) button;
-				dropDown.menuItem(modelName).click();
-				// hide drop down
-				dropDown.pressShortcut(KeyStroke.getInstance("ESC"));
-
-			}
-
-		return syncView.bot();
+	protected void setEnabledModelProvider(String modelProviderId) {
+		ITeamContentProviderManager contentProviderManager = TeamUI.getTeamContentProviderManager();
+		ITeamContentProviderDescriptor descriptor = contentProviderManager.getDescriptor(modelProviderId);
+		contentProviderManager.setEnabledDescriptors(new ITeamContentProviderDescriptor[] { descriptor });
 	}
 
 	// based on LocalRepositoryTestCase#createProjectAndCommitToRepository(String)
@@ -253,36 +223,6 @@ public abstract class AbstractSynchronizeViewTest extends
 		new ConnectProviderOperation(firstProject, gitDir).execute(null);
 	}
 
-	protected SWTBotEditor getCompareEditorForFileInGitChangeSet(
-			String fileName,
-			boolean includeLocalChanges) {
-		SWTBotTree syncViewTree = bot.viewByTitle("Synchronize").bot().tree();
-
-		SWTBotTreeItem rootTree;
-		if (includeLocalChanges)
-			rootTree = waitForNodeWithText(syncViewTree,
-					GitModelWorkingTree_workingTree);
-		else
-			rootTree = waitForNodeWithText(syncViewTree, TEST_COMMIT_MSG);
-
-		SWTBotTreeItem projNode = waitForNodeWithText(rootTree, PROJ1);
-		return getCompareEditor(projNode, fileName);
-	}
-
-	protected SWTBotEditor getCompareEditorForNonWorkspaceFileInGitChangeSet(
-			final String fileName) {
-		SWTBotTree syncViewTree = bot.viewByTitle("Synchronize").bot().tree();
-
-		SWTBotTreeItem rootTree = waitForNodeWithText(syncViewTree,
-					GitModelWorkingTree_workingTree);
-		waitForNodeWithText(rootTree, fileName).doubleClick();
-
-		SWTBotEditor editor = bot
-				.editor(new CompareEditorTitleMatcher(fileName));
-
-		return editor;
-	}
-
 	protected SWTBotTreeItem waitForNodeWithText(SWTBotTree tree, String name) {
 		waitUntilTreeHasNodeContainsText(bot, tree, name, 10000);
 		return getTreeItemContainingText(tree.getAllItems(), name).expand();
@@ -294,33 +234,6 @@ public abstract class AbstractSynchronizeViewTest extends
 		return getTreeItemContainingText(tree.getItems(), name).expand();
 	}
 
-	protected SWTBotEditor getCompareEditorForFileInWorkspaceModel()
-			throws Exception {
-		SWTBotTree syncViewTree = setPresentationModel("Workspace").tree();
-		SWTBotTreeItem projectTree = waitForNodeWithText(syncViewTree, PROJ1);
-		return getCompareEditor(projectTree, FILE1);
-	}
-
-	protected SWTBotEditor getCompareEditorForFileInGitChangeSetModel()
-			throws Exception {
-		SWTBotTree syncViewTree = setPresentationModel("Git Commits")
-				.tree();
-		SWTBotTreeItem commitNode = syncViewTree.getAllItems()[0];
-		commitNode.expand();
-		SWTBotTreeItem projectTree = waitForNodeWithText(commitNode, PROJ1);
-		return getCompareEditor(projectTree, FILE1);
-	}
-
-	protected SWTBotEditor getCompareEditorForFileInWorspaceModel(
-			String fileName) {
-		SWTBotTree syncViewTree = bot.viewByTitle("Synchronize").bot().tree();
-
-		SWTBotTreeItem projNode = waitForNodeWithText(syncViewTree, PROJ1);
-		SWTBotEditor editor = getCompareEditor(projNode, fileName);
-
-		return editor;
-	}
-
 	private static void createAndCommitDotGitignore() throws CoreException,
 			UnsupportedEncodingException {
 		IProject secondPoject = ResourcesPlugin.getWorkspace().getRoot()
@@ -344,14 +257,14 @@ public abstract class AbstractSynchronizeViewTest extends
 	protected void commit(String projectName) throws InterruptedException {
 		showDialog(projectName, "Team", CommitAction_commit);
 
-		bot.shell(CommitDialog_CommitChanges).bot().activeShell();
-		bot.styledText(0).setText(TEST_COMMIT_MSG);
-		bot.toolbarButtonWithTooltip(CommitDialog_SelectAll).click();
-		bot.button(CommitDialog_Commit).click();
+		SWTBot shellBot = bot.shell(CommitDialog_CommitChanges).bot();
+		shellBot.styledText(0).setText(TEST_COMMIT_MSG);
+		shellBot.toolbarButtonWithTooltip(CommitDialog_SelectAll).click();
+		shellBot.button(CommitDialog_Commit).click();
 		TestUtil.joinJobs(JobFamilies.COMMIT);
 	}
 
-	private SWTBotEditor getCompareEditor(SWTBotTreeItem projectNode,
+	protected SWTBotEditor getCompareEditor(SWTBotTreeItem projectNode,
 			final String fileName) {
 		SWTBotTreeItem folderNode = waitForNodeWithText(projectNode, FOLDER);
 		waitForNodeWithText(folderNode, fileName).doubleClick();
@@ -366,7 +279,6 @@ public abstract class AbstractSynchronizeViewTest extends
 
 	private static void showDialog(String projectName, String... cmd) {
 		SWTBot packageExplorerBot = bot.viewByTitle("Package Explorer").bot();
-		packageExplorerBot.activeShell();
 		SWTBotTree tree = packageExplorerBot.tree();
 
 		// EGit decorates the project node shown in the package explorer. The
@@ -387,18 +299,22 @@ public abstract class AbstractSynchronizeViewTest extends
 				return item;
 			}
 
-		throw new RuntimeException("Poject with name " + projectName +
+		throw new RuntimeException("Project with name " + projectName +
 				" was not found in given tree");
 	}
 
 	private SWTBotTreeItem getTreeItemContainingText(SWTBotTreeItem[] items,
 			String text) {
-		for (SWTBotTreeItem item : items)
+		List<String> existingItems = new ArrayList<String>();
+		for (SWTBotTreeItem item : items) {
 			if (item.getText().contains(text))
 				return item;
+			existingItems.add(item.getText());
+		}
 
 		throw new WidgetNotFoundException(
-					"Tree item elment containing text: test commit was not found");
+				"Tree item element containing text \"" + text
+						+ "\" was not found. Existing tree items:\n"
+						+ StringUtils.join(existingItems, "\n"));
 	}
-
 }
