@@ -1,5 +1,5 @@
 /*******************************************************************************
- * Copyright (C) 2010, Dariusz Luksza <dariusz@luksza.org>
+ * Copyright (C) 2010, 2013 Dariusz Luksza <dariusz@luksza.org> and others.
  *
  * All rights reserved. This program and the accompanying materials
  * are made available under the terms of the Eclipse Public License v1.0
@@ -31,6 +31,7 @@ import org.eclipse.core.resources.IFile;
 import org.eclipse.core.resources.IProject;
 import org.eclipse.core.resources.IResource;
 import org.eclipse.core.resources.ResourcesPlugin;
+import org.eclipse.core.resources.mapping.ModelProvider;
 import org.eclipse.core.runtime.jobs.Job;
 import org.eclipse.swt.widgets.Label;
 import org.eclipse.swtbot.eclipse.finder.widgets.SWTBotEditor;
@@ -40,15 +41,20 @@ import org.eclipse.swtbot.swt.finder.widgets.SWTBotStyledText;
 import org.eclipse.swtbot.swt.finder.widgets.SWTBotTree;
 import org.eclipse.swtbot.swt.finder.widgets.SWTBotTreeItem;
 import org.hamcrest.Matcher;
+import org.junit.Before;
 import org.junit.Ignore;
 import org.junit.Test;
 
 public class SynchronizeViewWorkspaceModelTest extends AbstractSynchronizeViewTest {
 
+	@Before
+	public void setUpEnabledModelProvider() {
+		setEnabledModelProvider(ModelProvider.RESOURCE_MODEL_PROVIDER_ID);
+	}
+
 	@Test
 	public void shouldReturnNoChanges() throws Exception {
 		// given
-		resetRepositoryToCreateInitialTag();
 		changeFilesInProject();
 
 		// when
@@ -68,7 +74,6 @@ public class SynchronizeViewWorkspaceModelTest extends AbstractSynchronizeViewTe
 	@Test
 	public void shouldReturnListOfChanges() throws Exception {
 		// given
-		resetRepositoryToCreateInitialTag();
 		changeFilesInProject();
 
 		// when
@@ -83,7 +88,6 @@ public class SynchronizeViewWorkspaceModelTest extends AbstractSynchronizeViewTe
 	@Test
 	public void shouldCompareBranchAgainstTag() throws Exception {
 		// given
-		resetRepositoryToCreateInitialTag();
 		makeChangesAndCommit(PROJ1);
 
 		// when
@@ -97,7 +101,6 @@ public class SynchronizeViewWorkspaceModelTest extends AbstractSynchronizeViewTe
 	@Test
 	public void shouldCompareTagAgainstTag() throws Exception {
 		// given
-		resetRepositoryToCreateInitialTag();
 		makeChangesAndCommit(PROJ1);
 		createTag("v0.1");
 
@@ -109,10 +112,10 @@ public class SynchronizeViewWorkspaceModelTest extends AbstractSynchronizeViewTe
 		assertEquals(1, syncViewTree.getAllItems().length);
 	}
 
-	@Test public void shouldOpenCompareEditor()
+	@Test
+	public void shouldOpenCompareEditor()
 			throws Exception {
 		// given
-		resetRepositoryToCreateInitialTag();
 		makeChangesAndCommit(PROJ1);
 		changeFilesInProject();
 
@@ -120,14 +123,14 @@ public class SynchronizeViewWorkspaceModelTest extends AbstractSynchronizeViewTe
 		launchSynchronization(HEAD, INITIAL_TAG, true);
 
 		// then
-		SWTBot compare = getCompareEditorForFileInWorspaceModel(FILE1)
+		SWTBot compare = getCompareEditorForFileInWorkspaceModel(FILE1)
 				.bot();
 		assertNotNull(compare);
 	}
 
-	@Test public void shouldListFileDeletedChange() throws Exception {
+	@Test
+	public void shouldListFileDeletedChange() throws Exception {
 		// given
-		resetRepositoryToCreateInitialTag();
 		deleteFileAndCommit(PROJ1);
 
 		// when
@@ -147,7 +150,8 @@ public class SynchronizeViewWorkspaceModelTest extends AbstractSynchronizeViewTe
 		assertEquals("test.txt", fileTree.getText());
 	}
 
-	@Test public void shouldSynchronizeInEmptyRepository() throws Exception {
+	@Test
+	public void shouldSynchronizeInEmptyRepository() throws Exception {
 		// given
 		createEmptyRepository();
 
@@ -169,15 +173,15 @@ public class SynchronizeViewWorkspaceModelTest extends AbstractSynchronizeViewTe
 		assertEquals(FILE2, fileTree.getText());
 	}
 
-	@Test public void shouldExchangeCompareEditorSidesBetweenIncomingAndOutgoingChanges()
+	@Test
+	public void shouldExchangeCompareEditorSidesBetweenIncomingAndOutgoingChanges()
 			throws Exception {
 		// given
-		resetRepositoryToCreateInitialTag();
 		makeChangesAndCommit(PROJ1);
 
 		// compare HEAD against tag
 		launchSynchronization(HEAD, INITIAL_TAG, false);
-		SWTBotEditor compEditor = getCompareEditorForFileInWorspaceModel(
+		SWTBotEditor compEditor = getCompareEditorForFileInWorkspaceModel(
 				FILE1);
 		SWTBot outgoingCompare = compEditor.bot();
 		// save left value from compare editor
@@ -191,7 +195,7 @@ public class SynchronizeViewWorkspaceModelTest extends AbstractSynchronizeViewTe
 		launchSynchronization(INITIAL_TAG, HEAD, false);
 
 		// then
-		SWTBot incomingComp = getCompareEditorForFileInWorspaceModel(
+		SWTBot incomingComp = getCompareEditorForFileInWorkspaceModel(
 				FILE1).bot();
 		String incomingLeft = incomingComp.styledText(0).getText();
 		String incomingRight = incomingComp.styledText(1).getText();
@@ -201,10 +205,10 @@ public class SynchronizeViewWorkspaceModelTest extends AbstractSynchronizeViewTe
 		assertThat(outgoingRight, equalTo(incomingLeft));
 	}
 
-	@Test public void shouldNotShowIgnoredFiles()
+	@Test
+	public void shouldNotShowIgnoredFiles()
 			throws Exception {
 		// given
-		resetRepositoryToCreateInitialTag();
 		String ignoredName = "to-be-ignored.txt";
 
 		IProject proj = ResourcesPlugin.getWorkspace().getRoot()
@@ -234,7 +238,6 @@ public class SynchronizeViewWorkspaceModelTest extends AbstractSynchronizeViewTe
 	public void shouldRefreshSyncResultAfterWorkspaceChange() throws Exception {
 		// given
 		String newFileName = "new.txt";
-		resetRepositoryToCreateInitialTag();
 		launchSynchronization(INITIAL_TAG, HEAD, true);
 		IProject proj = ResourcesPlugin.getWorkspace().getRoot()
 				.getProject(PROJ1);
@@ -258,7 +261,6 @@ public class SynchronizeViewWorkspaceModelTest extends AbstractSynchronizeViewTe
 	@Test
 	public void shouldRefreshSyncResultAfterRepositoryChange() throws Exception {
 		// given
-		resetRepositoryToCreateInitialTag();
 		changeFilesInProject();
 		launchSynchronization(HEAD, HEAD, true);
 
@@ -336,4 +338,14 @@ public class SynchronizeViewWorkspaceModelTest extends AbstractSynchronizeViewTe
 		assertNotSame(left, right);
 	}
 
+	protected SWTBotEditor getCompareEditorForFileInWorkspaceModel(
+			String fileName) {
+		SWTBotTree syncViewTree = bot.viewByTitle("Synchronize").bot().tree();
+
+		SWTBotTreeItem projNode = waitForNodeWithText(syncViewTree, PROJ1);
+		SWTBotEditor editor = getCompareEditor(projNode, fileName);
+
+		return editor;
+	}
+
 }
