@@ -2,6 +2,7 @@
  * Copyright (C) 2007,2010 Robin Rosenberg <robin.rosenberg@dewire.com>
  * Copyright (C) 2008, Shawn O. Pearce <spearce@spearce.org>
  * Copyright (C) 2010, Mathias Kinzler <mathias.kinzler@sap.com>
+ * Copyright (C) 2012, Matthias Sohn <matthias.sohn@sap.com>
  *
  * All rights reserved. This program and the accompanying materials
  * are made available under the terms of the Eclipse Public License v1.0
@@ -107,7 +108,24 @@ public class Activator extends AbstractUIPlugin implements DebugOptionsListener
 	 */
 	public static void handleError(String message, Throwable throwable,
 			boolean show) {
-		IStatus status = new Status(IStatus.ERROR, getPluginId(), message,
+		handleIssue(IStatus.ERROR, message, throwable, show);
+	}
+
+	/**
+	 * Handle an issue. The issue is logged. If <code>show</code> is
+	 * <code>true</code> the issue is shown to the user.
+	 *
+	 * @param severity
+	 *            status severity, use constants defined in {@link IStatus}
+	 * @param message
+	 *            a localized message
+	 * @param throwable
+	 * @param show
+	 * @since 2.2
+	 */
+	public static void handleIssue(int severity, String message, Throwable throwable,
+			boolean show) {
+		IStatus status = new Status(severity, getPluginId(), message,
 				throwable);
 		int style = StatusManager.LOG;
 		if (show)
