@@ -1,5 +1,5 @@
 /*******************************************************************************
- * Copyright (C) 2011, Mathias Kinzler <mathias.kinzler@sap.com>
+ * Copyright (C) 2011, 2012 Mathias Kinzler <mathias.kinzler@sap.com> and others.
  *
  * All rights reserved. This program and the accompanying materials
  * are made available under the terms of the Eclipse Public License v1.0
@@ -25,6 +25,7 @@ import org.eclipse.egit.ui.internal.GitCompareFileRevisionEditorInput;
 import org.eclipse.egit.ui.internal.dialogs.CompareTreeView;
 import org.eclipse.egit.ui.internal.history.CommitSelectionDialog;
 import org.eclipse.jface.window.Window;
+import org.eclipse.jgit.lib.Constants;
 import org.eclipse.jgit.lib.ObjectId;
 import org.eclipse.jgit.lib.Repository;
 import org.eclipse.jgit.revwalk.RevCommit;
@@ -42,12 +43,12 @@ public class CompareWithCommitActionHandler extends RepositoryActionHandler {
 
 	public Object execute(ExecutionEvent event) throws ExecutionException {
 		final Repository repo = getRepository(true, event);
-		IResource[] resources = getSelectedResources(event);
 		if (repo == null)
 			return null;
+		IResource[] resources = getSelectedResources(event);
 
 		CommitSelectionDialog dlg = new CommitSelectionDialog(getShell(event),
-				repo);
+				repo, resources);
 		if (dlg.open() != Window.OK)
 			return null;
 
@@ -58,11 +59,16 @@ public class CompareWithCommitActionHandler extends RepositoryActionHandler {
 					.createFileElement(baseFile);
 
 			final ITypedElement next;
+			final ITypedElement ancestor;
 			try {
 				RepositoryMapping mapping = RepositoryMapping
 						.getMapping(resources[0]);
 				next = getElementForCommit(mapping.getRepository(), mapping
 						.getRepoRelativePath(baseFile), dlg.getCommitId());
+
+				ancestor =  CompareUtils.getFileRevisionTypedElementForCommonAncestor(mapping
+						.getRepoRelativePath(baseFile), repo.resolve(Constants.HEAD),
+						dlg.getCommitId(), repo);
 			} catch (IOException e) {
 				Activator.handleError(
 						UIText.CompareWithIndexAction_errorOnAddToIndex, e,
@@ -71,7 +77,7 @@ public class CompareWithCommitActionHandler extends RepositoryActionHandler {
 			}
 
 			final GitCompareFileRevisionEditorInput in = new GitCompareFileRevisionEditorInput(
-					base, next, null);
+					base, next, ancestor, null);
 			in.getCompareConfiguration()
 					.setRightLabel(dlg.getCommitId().name());
 			CompareUI.openCompareEditor(in);
@@ -100,6 +106,6 @@ public class CompareWithCommitActionHandler extends RepositoryActionHandler {
 
 	@Override
 	public boolean isEnabled() {
-		return getRepository() != null;
+		return selectionMapsToSingleRepository();
 	}
 }
