@@ -1,5 +1,5 @@
 /*******************************************************************************
- * Copyright (C) 2011, Dariusz Luksza <dariusz@luksza.org>
+ * Copyright (C) 2011, 2012 Dariusz Luksza <dariusz@luksza.org> and others.
  *
  * All rights reserved. This program and the accompanying materials
  * are made available under the terms of the Eclipse Public License v1.0
@@ -83,6 +83,8 @@ public class SynchronizeWorkspaceActionHandler extends RepositoryActionHandler {
 
 		for (IResource resource : resources) {
 			RepositoryMapping rm = RepositoryMapping.getMapping(resource);
+			if (rm == null)
+				continue; // Linked resources may not be in a repo
 			if (resource instanceof IProject)
 				result.put(rm.getRepository(), new HashSet<IContainer>());
 			else if (resource instanceof IContainer) {
