@@ -7,6 +7,7 @@
  * Copyright (C) 2011, Mathias Kinzler <mathias.kinzler@sap.com>
  * Copyright (C) 2012, Daniel Megert <daniel_megert@ch.ibm.com>
  * Copyright (C) 2012, Robin Stocker <robin@nibor.org>
+ * Copyright (C) 2012, IBM Corporation (Markus Keller <markus_keller@ch.ibm.com>)
  *
  * All rights reserved. This program and the accompanying materials
  * are made available under the terms of the Eclipse Public License v1.0
@@ -40,7 +41,9 @@ import org.eclipse.egit.ui.UIIcons;
 import org.eclipse.egit.ui.UIPreferences;
 import org.eclipse.egit.ui.UIText;
 import org.eclipse.egit.ui.UIUtils;
+import org.eclipse.egit.ui.internal.CachedCheckboxTreeViewer;
 import org.eclipse.egit.ui.internal.CompareUtils;
+import org.eclipse.egit.ui.internal.FilteredCheckboxTree;
 import org.eclipse.egit.ui.internal.commit.CommitHelper;
 import org.eclipse.egit.ui.internal.commit.CommitMessageHistory;
 import org.eclipse.egit.ui.internal.commit.CommitProposalProcessor;
@@ -56,18 +59,15 @@ import org.eclipse.jface.dialogs.TitleAreaDialog;
 import org.eclipse.jface.layout.GridDataFactory;
 import org.eclipse.jface.layout.GridLayoutFactory;
 import org.eclipse.jface.preference.IPreferenceStore;
-import org.eclipse.jface.preference.PreferenceDialog;
 import org.eclipse.jface.resource.ImageDescriptor;
 import org.eclipse.jface.resource.JFaceResources;
 import org.eclipse.jface.resource.LocalResourceManager;
 import org.eclipse.jface.resource.ResourceManager;
 import org.eclipse.jface.text.DocumentEvent;
 import org.eclipse.jface.text.IDocumentListener;
-import org.eclipse.jface.viewers.ArrayContentProvider;
 import org.eclipse.jface.viewers.BaseLabelProvider;
 import org.eclipse.jface.viewers.CellLabelProvider;
 import org.eclipse.jface.viewers.CheckStateChangedEvent;
-import org.eclipse.jface.viewers.CheckboxTableViewer;
 import org.eclipse.jface.viewers.ColumnLabelProvider;
 import org.eclipse.jface.viewers.ColumnViewerToolTipSupport;
 import org.eclipse.jface.viewers.DecoratingStyledCellLabelProvider;
@@ -77,11 +77,10 @@ import org.eclipse.jface.viewers.ICheckStateListener;
 import org.eclipse.jface.viewers.IDecoration;
 import org.eclipse.jface.viewers.IStructuredSelection;
 import org.eclipse.jface.viewers.StyledString;
-import org.eclipse.jface.viewers.TableViewerColumn;
+import org.eclipse.jface.viewers.TreeViewerColumn;
 import org.eclipse.jface.viewers.Viewer;
 import org.eclipse.jface.viewers.ViewerComparator;
 import org.eclipse.jface.viewers.ViewerFilter;
-import org.eclipse.jface.window.Window;
 import org.eclipse.jgit.api.AddCommand;
 import org.eclipse.jgit.api.Git;
 import org.eclipse.jgit.lib.Constants;
@@ -112,14 +111,18 @@ import org.eclipse.swt.widgets.TableColumn;
 import org.eclipse.swt.widgets.Text;
 import org.eclipse.swt.widgets.ToolBar;
 import org.eclipse.swt.widgets.ToolItem;
+import org.eclipse.swt.widgets.Tree;
+import org.eclipse.swt.widgets.TreeColumn;
 import org.eclipse.team.core.RepositoryProvider;
 import org.eclipse.ui.ISharedImages;
 import org.eclipse.ui.PlatformUI;
+import org.eclipse.ui.dialogs.PatternFilter;
 import org.eclipse.ui.dialogs.PreferencesUtil;
 import org.eclipse.ui.forms.IFormColors;
 import org.eclipse.ui.forms.widgets.ExpandableComposite;
 import org.eclipse.ui.forms.widgets.FormToolkit;
 import org.eclipse.ui.forms.widgets.Section;
+import org.eclipse.ui.model.BaseWorkbenchContentProvider;
 
 /**
  * Dialog is shown to user when they request to commit files. Changes in the
@@ -131,6 +134,29 @@ public class CommitDialog extends TitleAreaDialog {
 		return org.eclipse.egit.ui.Activator.getDefault().getPreferenceStore();
 	}
 
+	static class CommitFileContentProvider extends BaseWorkbenchContentProvider {
+		@Override
+		public Object[] getElements(Object element) {
+			if (element instanceof Object[])
+				return (Object[]) element;
+			if (element instanceof Collection)
+				return ((Collection) element).toArray();
+			return new Object[0];
+		}
+
+		public Object[] getChildren(Object parentElement) {
+			return new Object[0];
+		}
+
+		public Object getParent(Object element) {
+			return null;
+		}
+
+		public boolean hasChildren(Object element) {
+			return false;
+		}
+	}
+
 	static class CommitStatusLabelProvider extends BaseLabelProvider implements
 			IStyledLabelProvider {
 
@@ -301,6 +327,11 @@ public class CommitDialog extends TitleAreaDialog {
 
 	private static final String SHOW_UNTRACKED_PREF = "CommitDialog.showUntracked"; //$NON-NLS-1$
 
+	/**
+	 * A constant used for the 'commit and push button' button
+	 */
+	public static final int COMMIT_AND_PUSH_ID = 30;
+
 	FormToolkit toolkit;
 
 	CommitMessageComponent commitMessageComponent;
@@ -319,12 +350,14 @@ public class CommitDialog extends TitleAreaDialog {
 
 	ToolItem showUntrackedItem;
 
-	CheckboxTableViewer filesViewer;
+	CachedCheckboxTreeViewer filesViewer;
 
 	Section filesSection;
 
 	Button commitButton;
 
+	Button commitAndPushButton;
+
 	ArrayList<CommitItem> items = new ArrayList<CommitItem>();
 
 	private String commitMessage = null;
@@ -354,7 +387,7 @@ public class CommitDialog extends TitleAreaDialog {
 
 	private Repository repository;
 
-	private boolean pushEnabled = false;
+	private boolean isPushRequested = false;
 
 	/**
 	 * @param parentShell
@@ -510,16 +543,17 @@ public class CommitDialog extends TitleAreaDialog {
 	}
 
 	/**
-	 * Returns whether we are pushing after the commit
-	 * @return pushing
+	 * @return true if push shall be executed
 	 */
-	public boolean isPushEnabled() {
-		return pushEnabled;
+	public boolean isPushRequested() {
+		return isPushRequested;
 	}
 
 	@Override
 	protected void createButtonsForButtonBar(Composite parent) {
 		toolkit.adapt(parent, false, false);
+		commitAndPushButton = createButton(parent, COMMIT_AND_PUSH_ID,
+				UIText.CommitDialog_CommitAndPush, false);
 		commitButton = createButton(parent, IDialogConstants.OK_ID,
 				UIText.CommitDialog_Commit, true);
 		createButton(parent, IDialogConstants.CANCEL_ID,
@@ -527,6 +561,16 @@ public class CommitDialog extends TitleAreaDialog {
 		updateMessage();
 	}
 
+	protected void buttonPressed(int buttonId) {
+		if (IDialogConstants.OK_ID == buttonId)
+			okPressed();
+		else if (COMMIT_AND_PUSH_ID == buttonId) {
+			isPushRequested = true;
+			okPressed();
+		} else if (IDialogConstants.CANCEL_ID == buttonId)
+			cancelPressed();
+	}
+
 	@Override
 	protected Control createButtonBar(Composite parent) {
 		toolkit.adapt(parent, false, false);
@@ -565,11 +609,8 @@ public class CommitDialog extends TitleAreaDialog {
 
 			public void widgetSelected(SelectionEvent e) {
 				String[] pages = new String[] { UIPreferences.PAGE_COMMIT_PREFERENCES };
-				PreferenceDialog dialog = PreferencesUtil
-						.createPreferenceDialogOn(getShell(), pages[0], pages,
-								null);
-				if (Window.OK == dialog.open())
-					commitText.reconfigure();
+				PreferencesUtil.createPreferenceDialogOn(getShell(), pages[0],
+						pages, null).open();
 			}
 
 		});
@@ -765,44 +806,63 @@ public class CommitDialog extends TitleAreaDialog {
 
 		filesSection.setTextClient(filesToolbar);
 
-		Table resourcesTable = toolkit.createTable(filesArea, SWT.H_SCROLL
-				| SWT.V_SCROLL | SWT.FULL_SELECTION | SWT.MULTI | SWT.CHECK);
-		resourcesTable.setData(FormToolkit.KEY_DRAW_BORDER,
+		PatternFilter patternFilter = new PatternFilter() {
+			@Override
+			protected boolean isLeafMatch(Viewer viewer, Object element) {
+				if(element instanceof CommitItem) {
+					CommitItem commitItem = (CommitItem) element;
+					return wordMatches(commitItem.path);
+				}
+				return super.isLeafMatch(viewer, element);
+			}
+		};
+		patternFilter.setIncludeLeadingWildcard(true);
+		FilteredCheckboxTree resourcesTreeComposite = new FilteredCheckboxTree(
+				filesArea, toolkit, SWT.FULL_SELECTION, patternFilter);
+		Tree resourcesTree = resourcesTreeComposite.getViewer().getTree();
+		resourcesTree.setData(FormToolkit.KEY_DRAW_BORDER,
 				FormToolkit.TREE_BORDER);
-		resourcesTable.setLayoutData(GridDataFactory.fillDefaults()
+		resourcesTreeComposite.setLayoutData(GridDataFactory.fillDefaults()
 				.hint(600, 200).grab(true, true).create());
 
-		resourcesTable.addSelectionListener(new CommitItemSelectionListener());
+		resourcesTree.addSelectionListener(new CommitItemSelectionListener());
 
-		resourcesTable.setHeaderVisible(true);
-		TableColumn statCol = new TableColumn(resourcesTable, SWT.LEFT);
+		resourcesTree.setHeaderVisible(true);
+		TreeColumn statCol = new TreeColumn(resourcesTree, SWT.LEFT);
 		statCol.setText(UIText.CommitDialog_Status);
 		statCol.setWidth(150);
 		statCol.addSelectionListener(new HeaderSelectionListener(
 				CommitItem.Order.ByStatus));
 
-		TableColumn resourceCol = new TableColumn(resourcesTable, SWT.LEFT);
+		TreeColumn resourceCol = new TreeColumn(resourcesTree, SWT.LEFT);
 		resourceCol.setText(UIText.CommitDialog_Path);
 		resourceCol.setWidth(415);
 		resourceCol.addSelectionListener(new HeaderSelectionListener(
 				CommitItem.Order.ByFile));
 
-		filesViewer = new CheckboxTableViewer(resourcesTable);
-		new TableViewerColumn(filesViewer, statCol)
+		filesViewer = resourcesTreeComposite.getCheckboxTreeViewer();
+		new TreeViewerColumn(filesViewer, statCol)
 				.setLabelProvider(createStatusLabelProvider());
-		new TableViewerColumn(filesViewer, resourceCol)
+		new TreeViewerColumn(filesViewer, resourceCol)
 				.setLabelProvider(new CommitPathLabelProvider());
 		ColumnViewerToolTipSupport.enableFor(filesViewer);
-		filesViewer.setContentProvider(ArrayContentProvider.getInstance());
+		filesViewer.setContentProvider(new CommitFileContentProvider());
 		filesViewer.setUseHashlookup(true);
 		IDialogSettings settings = org.eclipse.egit.ui.Activator.getDefault()
 				.getDialogSettings();
-		if (settings.get(SHOW_UNTRACKED_PREF) != null)
+		if (settings.get(SHOW_UNTRACKED_PREF) != null) {
+			// note, no matter how the dialog settings are, if
+			// the preferences force us to include untracked files
+			// we must show them
 			showUntracked = Boolean.valueOf(settings.get(SHOW_UNTRACKED_PREF))
-					.booleanValue();
+					.booleanValue()
+					|| getPreferenceStore().getBoolean(
+							UIPreferences.COMMIT_DIALOG_INCLUDE_UNTRACKED);
+		}
+
 		filesViewer.addFilter(new CommitItemFilter());
 		filesViewer.setInput(items.toArray());
-		filesViewer.getTable().setMenu(getContextMenu());
+		filesViewer.getTree().setMenu(getContextMenu());
 		filesViewer.addCheckStateListener(new ICheckStateListener() {
 
 			public void checkStateChanged(CheckStateChangedEvent event) {
@@ -874,7 +934,6 @@ public class CommitDialog extends TitleAreaDialog {
 					updateFileSectionText();
 				}
 			});
-			filesViewer.setAllGrayed(true);
 			filesViewer.setAllChecked(true);
 		} else {
 			final boolean includeUntracked = getPreferenceStore().getBoolean(
@@ -890,32 +949,6 @@ public class CommitDialog extends TitleAreaDialog {
 			}
 		}
 
-		Section pushSection = toolkit.createSection(container,
-				ExpandableComposite.TITLE_BAR
-						| ExpandableComposite.CLIENT_INDENT);
-		GridDataFactory.fillDefaults().grab(true, true).applyTo(pushSection);
-		Composite pushArea = toolkit.createComposite(pushSection);
-		pushSection.setClient(pushArea);
-		toolkit.paintBordersFor(pushArea);
-		GridLayoutFactory.fillDefaults().extendedMargins(2, 2, 2, 2)
-				.applyTo(pushArea);
-		pushSection.setText(UIText.CommitDialog_PushSectionTitle);
-		final Button pushCheckbox = toolkit.createButton(pushArea,
-				UIText.CommitDialog_PushUpstream, SWT.CHECK);
-		pushCheckbox.setSelection(getPreferenceStore().getBoolean(
-					UIPreferences.COMMIT_DIALOG_PUSH_UPSTREAM));
-		pushEnabled = getPreferenceStore().getBoolean(
-				UIPreferences.COMMIT_DIALOG_PUSH_UPSTREAM);
-		pushCheckbox.addSelectionListener(new SelectionAdapter() {
-
-			@Override
-			public void widgetSelected(SelectionEvent e) {
-				pushEnabled = pushCheckbox.getSelection();
-				getPreferenceStore().setValue(
-						UIPreferences.COMMIT_DIALOG_PUSH_UPSTREAM, pushEnabled);
-			}
-		});
-
 		applyDialogFont(container);
 		statCol.pack();
 		resourceCol.pack();
@@ -985,8 +1018,10 @@ public class CommitDialog extends TitleAreaDialog {
 		}
 
 		setMessage(message, type);
-		commitButton.setEnabled(type == IMessageProvider.WARNING
-				|| type == IMessageProvider.NONE);
+		boolean commitEnabled = type == IMessageProvider.WARNING
+				|| type == IMessageProvider.NONE;
+		commitButton.setEnabled(commitEnabled);
+		commitAndPushButton.setEnabled(commitEnabled);
 	}
 
 	private boolean isCommitWithoutFilesAllowed() {
@@ -1010,14 +1045,14 @@ public class CommitDialog extends TitleAreaDialog {
 	private void updateFileSectionText() {
 		filesSection.setText(MessageFormat.format(UIText.CommitDialog_Files,
 				Integer.valueOf(filesViewer.getCheckedElements().length),
-				Integer.valueOf(filesViewer.getTable().getItemCount())));
+				Integer.valueOf(filesViewer.getTree().getItemCount())));
 	}
 
 	private Menu getContextMenu() {
 		if (!allowToChangeSelection)
 			return null;
-		final Menu menu = new Menu(filesViewer.getTable());
-		filesViewer.getTable().addDisposeListener(new DisposeListener() {
+		final Menu menu = new Menu(filesViewer.getTree());
+		filesViewer.getTree().addDisposeListener(new DisposeListener() {
 
 			public void widgetDisposed(DisposeEvent e) {
 				menu.dispose();
