@@ -6,6 +6,8 @@
  * Copyright (C) 2007, Shawn O. Pearce <spearce@spearce.org>
  * Copyright (C) 2011, Mathias Kinzler <mathias.kinzler@sap.com>
  * Copyright (C) 2011, Jens Baumgart <jens.baumgart@sap.com>
+ * Copyright (C) 2012, IBM Corporation (Markus Keller <markus_keller@ch.ibm.com>)
+ * Copyright (C) 2012, Robin Stocker <robin@nibor.org>
  *
  * All rights reserved. This program and the accompanying materials
  * are made available under the terms of the Eclipse Public License v1.0
@@ -124,6 +126,8 @@ public class CommitMessageComponent {
 
 	private String commitMessage = null;
 
+	private String commitMessageBeforeAmending = EMPTY_STRING;
+
 	private String previousCommitMessage = EMPTY_STRING;
 
 	private String author = null;
@@ -181,7 +185,8 @@ public class CommitMessageComponent {
 	public void resetState() {
 		originalChangeId = null;
 		commitMessage = null;
-		previousCommitMessage =EMPTY_STRING;
+		commitMessageBeforeAmending = EMPTY_STRING;
+		previousCommitMessage = EMPTY_STRING;
 		author = null;
 		previousAuthor = null;
 		committer = null;
@@ -195,7 +200,10 @@ public class CommitMessageComponent {
 	}
 
 	/**
-	 * @return The message the user entered
+	 * Returns the commit message, converting platform-specific line endings to
+	 * '\n' and hard-wrapping lines if necessary.
+	 *
+	 * @return the message
 	 */
 	public String getCommitMessage() {
 		commitMessage = commitText.getCommitMessage();
@@ -316,6 +324,7 @@ public class CommitMessageComponent {
 	 */
 	public void setAmendAllowed(boolean amendAllowed) {
 		this.amendAllowed = amendAllowed;
+		commitMessageBeforeAmending = EMPTY_STRING;
 	}
 
 	/**
@@ -326,9 +335,12 @@ public class CommitMessageComponent {
 		if (!selection) {
 			originalChangeId = null;
 			authorText.setText(author);
+			commitText.setText(commitMessageBeforeAmending);
+			commitMessageBeforeAmending = EMPTY_STRING;
 		} else {
 			getHeadCommitInfo();
 			saveOriginalChangeId();
+			commitMessageBeforeAmending = commitText.getText();
 			commitText.setText(previousCommitMessage);
 			if (previousAuthor != null)
 				authorText.setText(previousAuthor);
@@ -347,7 +359,7 @@ public class CommitMessageComponent {
 	 *
 	 */
 	public void updateStateFromUI() {
-		commitMessage = commitText.getCommitMessage();
+		commitMessage = commitText.getText();
 		author = authorText.getText().trim();
 		committer = committerText.getText().trim();
 	}
@@ -527,7 +539,9 @@ public class CommitMessageComponent {
 		if (amending)
 			getHeadCommitInfo();
 
-		commitText.setText(calculateCommitMessage(filesToCommit));
+		String calculatedCommitMessage = calculateCommitMessage(filesToCommit);
+		boolean calculatedMessageHasChangeId = findOffsetOfChangeIdLine(calculatedCommitMessage) > 0;
+		commitText.setText(calculatedCommitMessage);
 		authorText.setText(getSafeString(author));
 		committerText.setText(getSafeString(committer));
 		if (amending) {
@@ -538,7 +552,8 @@ public class CommitMessageComponent {
 				originalChangeId = null;
 			}
 			refreshSignedOffBy();
-			refreshChangeIdText();
+			if (!calculatedMessageHasChangeId)
+				refreshChangeIdText();
 		}
 		updateSignedOffButton();
 		updateChangeIdButton();
@@ -647,7 +662,7 @@ public class CommitMessageComponent {
 					previousCommitMessage);
 			if (endOfChangeId < 0)
 				endOfChangeId = previousCommitMessage.length();
-			int sha1Offset = changeIdOffset + Text.DELIMITER.length() + "Change-Id: I".length(); //$NON-NLS-1$
+			int sha1Offset = changeIdOffset + "Change-Id: I".length(); //$NON-NLS-1$
 			try {
 				originalChangeId = ObjectId.fromString(previousCommitMessage
 						.substring(sha1Offset, endOfChangeId));
@@ -663,7 +678,7 @@ public class CommitMessageComponent {
 	}
 
 	private int findOffsetOfChangeIdLine(String message) {
-		return message.indexOf(Text.DELIMITER + "Change-Id: I"); //$NON-NLS-1$
+		return ChangeIdUtil.indexOfChangeId(message, Text.DELIMITER);
 	}
 
 	private void updateChangeIdButton() {
@@ -697,7 +712,7 @@ public class CommitMessageComponent {
 					cleanedText = text.substring(0, changeIdOffset);
 				else
 					cleanedText = text.substring(0, changeIdOffset)
-							+ text.substring(endOfChangeId);
+							+ text.substring(endOfChangeId + 1);
 				commitText.setText(cleanedText);
 			}
 		}
