@@ -1,5 +1,5 @@
 /*******************************************************************************
- * Copyright (c) 2011, 2012 SAP AG and others.
+ * Copyright (c) 2011-2013 SAP AG and others.
  *
  * All rights reserved. This program and the accompanying materials
  * are made available under the terms of the Eclipse Public License v1.0
@@ -8,6 +8,7 @@
  *
  * Contributors:
  *    Mathias Kinzler (SAP AG) - initial implementation
+ *    Robin Stocker <robin@nibor.org> - ignore linked resources
  *******************************************************************************/
 package org.eclipse.egit.ui.internal.dialogs;
 
@@ -43,6 +44,7 @@ import org.eclipse.egit.ui.Activator;
 import org.eclipse.egit.ui.UIIcons;
 import org.eclipse.egit.ui.UIPreferences;
 import org.eclipse.egit.ui.UIText;
+import org.eclipse.egit.ui.UIUtils;
 import org.eclipse.egit.ui.internal.CompareUtils;
 import org.eclipse.egit.ui.internal.EgitUiEditorUtils;
 import org.eclipse.egit.ui.internal.FileEditableRevision;
@@ -99,7 +101,6 @@ import org.eclipse.ui.IWorkbenchPage;
 import org.eclipse.ui.IWorkbenchWindow;
 import org.eclipse.ui.PlatformUI;
 import org.eclipse.ui.actions.ActionFactory.IWorkbenchAction;
-import org.eclipse.ui.actions.ContributionItemFactory;
 import org.eclipse.ui.model.IWorkbenchAdapter;
 import org.eclipse.ui.model.WorkbenchContentProvider;
 import org.eclipse.ui.model.WorkbenchLabelProvider;
@@ -115,7 +116,8 @@ import org.eclipse.ui.part.ViewPart;
  * If the input is an {@link IContainer}, the tree is similar to the tree shown
  * by the PackageExplorer (based on {@link WorkbenchLabelProvider} and
  * {@link WorkbenchContentProvider}, otherwise a simple tree representing files
- * and folders is used based on {@link PathNode} instances.
+ * and folders is used based on {@link PathNode} instances. Linked resources
+ * however are ignored and not listed as content.
  * <p>
  * The tree nodes are shown with icons for "Added", "Deleted", and
  * "Same Contents" for files. Files with same content can be hidden using a
@@ -123,6 +125,7 @@ import org.eclipse.ui.part.ViewPart;
  * <p>
  * This view can also show files and folders outside the Eclipse workspace when
  * a {@link Repository} is used as input.
+ * <p>
  */
 public class CompareTreeView extends ViewPart implements IMenuListener, IShowInSource {
 	/** The "magic" compare version to compare with the index */
@@ -217,14 +220,24 @@ public class CompareTreeView extends ViewPart implements IMenuListener, IShowInS
 		getViewSite().getActionBars().getToolBarManager().add(showEqualsAction);
 
 		IAction expandAllAction = new Action(
+				UIText.CompareTreeView_ExpandAllTooltip) {
+			@Override
+			public void run() {
+				tree.expandAll();
+			}
+		};
+		expandAllAction.setImageDescriptor(UIIcons.EXPAND_ALL);
+		getViewSite().getActionBars().getToolBarManager().add(expandAllAction);
+
+		IAction collapseAllAction = new Action(
 				UIText.CompareTreeView_CollapseAllTooltip) {
 			@Override
 			public void run() {
 				tree.collapseAll();
 			}
 		};
-		expandAllAction.setImageDescriptor(UIIcons.COLLAPSEALL);
-		getViewSite().getActionBars().getToolBarManager().add(expandAllAction);
+		collapseAllAction.setImageDescriptor(UIIcons.COLLAPSEALL);
+		getViewSite().getActionBars().getToolBarManager().add(collapseAllAction);
 	}
 
 	private void reactOnOpen(OpenEvent event) {
@@ -922,9 +935,12 @@ public class CompareTreeView extends ViewPart implements IMenuListener, IShowInS
 				children = super.getChildren(element);
 			List<Object> childList = new ArrayList<Object>(children.length);
 			for (Object child : children) {
-				IPath path = new Path(repositoryMapping
-						.getRepoRelativePath((IResource) child));
-				boolean isFile = ((IResource) child).getType() == IResource.FILE;
+				IResource childResource = (IResource) child;
+				if (childResource.isLinked())
+					continue;
+				IPath path = new Path(
+						repositoryMapping.getRepoRelativePath(childResource));
+				boolean isFile = childResource.getType() == IResource.FILE;
 
 				// each path that is not ignored creates an entry in either
 				// compareVersionMap or addedPaths, so we can check if a path
@@ -1138,10 +1154,8 @@ public class CompareTreeView extends ViewPart implements IMenuListener, IShowInS
 		if (openAction != null)
 			manager.appendToGroup(ICommonMenuConstants.GROUP_OPEN, openAction);
 
-		MenuManager showInSubMenu = new MenuManager(
-				UIText.CompareTreeView_ShowIn_label);
-		showInSubMenu.add(ContributionItemFactory.VIEWS_SHOW_IN
-				.create(getSite().getWorkbenchWindow()));
+		MenuManager showInSubMenu = UIUtils.createShowInMenu(
+				getSite().getWorkbenchWindow());
 		manager.appendToGroup(ICommonMenuConstants.GROUP_OPEN, showInSubMenu);
 	}
 
