@@ -2,6 +2,7 @@
  * Copyright (C) 2010, Benjamin Muskalla <bmuskalla@eclipsesource.com>
  * Copyright (C) 2011, Matthias Sohn <matthias.sohn@sap.com>
  * Copyright (C) 2011-2012, IBM Corporation
+ * Copyright (C) 2012, Robin Stocker <robin@nibor.org>
  *
  * All rights reserved. This program and the accompanying materials
  * are made available under the terms of the Eclipse Public License v1.0
@@ -14,12 +15,11 @@
  *******************************************************************************/
 package org.eclipse.egit.ui.internal.dialogs;
 
+import java.util.Arrays;
 import java.util.Collections;
 import java.util.Iterator;
-import java.util.LinkedList;
 import java.util.List;
 import java.util.Map;
-import java.util.regex.Pattern;
 
 import org.eclipse.core.runtime.Assert;
 import org.eclipse.core.runtime.IAdaptable;
@@ -65,20 +65,23 @@ import org.eclipse.jface.util.IPropertyChangeListener;
 import org.eclipse.jface.util.PropertyChangeEvent;
 import org.eclipse.jface.viewers.ISelectionChangedListener;
 import org.eclipse.jface.viewers.SelectionChangedEvent;
+import org.eclipse.jgit.util.IntList;
 import org.eclipse.swt.SWT;
+import org.eclipse.swt.custom.BidiSegmentEvent;
+import org.eclipse.swt.custom.BidiSegmentListener;
 import org.eclipse.swt.custom.StyledText;
 import org.eclipse.swt.events.DisposeEvent;
 import org.eclipse.swt.events.DisposeListener;
 import org.eclipse.swt.events.FocusEvent;
 import org.eclipse.swt.events.FocusListener;
-import org.eclipse.swt.events.ModifyEvent;
-import org.eclipse.swt.events.ModifyListener;
 import org.eclipse.swt.graphics.GC;
 import org.eclipse.swt.graphics.Image;
 import org.eclipse.swt.graphics.Point;
+import org.eclipse.swt.graphics.Rectangle;
 import org.eclipse.swt.layout.FillLayout;
 import org.eclipse.swt.widgets.Composite;
 import org.eclipse.swt.widgets.Display;
+import org.eclipse.swt.widgets.Layout;
 import org.eclipse.ui.ActiveShellExpression;
 import org.eclipse.ui.IWorkbenchCommandConstants;
 import org.eclipse.ui.PlatformUI;
@@ -101,9 +104,6 @@ public class SpellcheckableMessageArea extends Composite {
 
 	static final int MAX_LINE_WIDTH = 72;
 
-	// Symbol or Punctuation, Dash or Punctuation, Other
-	private static final Pattern STARTS_WITH_SYMBOL = Pattern.compile("[\\p{S}\\p{Pd}\\p{Po}].*"); //$NON-NLS-1$
-
 	private static class TextViewerAction extends Action implements IUpdate {
 
 		private int fOperationCode= -1;
@@ -212,7 +212,14 @@ public class SpellcheckableMessageArea extends Composite {
 
 	private final SourceViewer sourceViewer;
 
-	private ModifyListener hardWrapModifyListener;
+	private BidiSegmentListener hardWrapSegmentListener;
+
+	// XXX: workaround for https://bugs.eclipse.org/400727
+	private int brokenBidiPlatformTextWidth;
+
+	private ActionHandler quickFixActionHandler;
+
+	private ActionHandler contentAssistActionHandler;
 
 	/**
 	 * @param parent
@@ -256,21 +263,32 @@ public class SpellcheckableMessageArea extends Composite {
 		Point size = getTextWidget().computeSize(textWidth, textHeight);
 		getTextWidget().setSize(size);
 
+		computeBrokenBidiPlatformTextWidth(size.x);
+
 		getTextWidget().setEditable(!readOnly);
 
 		createMarginPainter();
 
 		configureHardWrap();
+		final IPropertyChangeListener propertyChangeListener = new IPropertyChangeListener() {
+			public void propertyChange(PropertyChangeEvent event) {
+				if (UIPreferences.COMMIT_DIALOG_HARD_WRAP_MESSAGE.equals(event.getProperty())) {
+					getDisplay().asyncExec(new Runnable() {
+						public void run() {
+							configureHardWrap();
+							if (brokenBidiPlatformTextWidth != -1) {
+								layout();
+							}
+						}
+					});
+				}
+			}
+		};
+		Activator.getDefault().getPreferenceStore().addPropertyChangeListener(propertyChangeListener);
 
 		final SourceViewerDecorationSupport support = configureAnnotationPreferences();
 		if (isEditable(sourceViewer)) {
-			final IHandlerActivation handlerActivation = installQuickFixActionHandler();
-			getTextWidget().addDisposeListener(new DisposeListener() {
-
-				public void widgetDisposed(DisposeEvent e) {
-					getHandlerService().deactivateHandler(handlerActivation);
-				}
-			});
+			quickFixActionHandler = createQuickFixActionHandler(sourceViewer);
 		}
 
 		Document document = new Document(initialText);
@@ -294,17 +312,8 @@ public class SpellcheckableMessageArea extends Composite {
 					return null;
 				IContentAssistant assistant = createContentAssistant(viewer);
 				// Add content assist proposal handler if assistant exists
-				if (assistant != null) {
-					final IHandlerActivation activation = installContentAssistActionHandler();
-					viewer.getTextWidget().addDisposeListener(
-							new DisposeListener() {
-
-								public void widgetDisposed(DisposeEvent e) {
-									getHandlerService().deactivateHandler(
-											activation);
-								}
-							});
-				}
+				if (assistant != null)
+					contentAssistActionHandler = createContentAssistActionHandler(sourceViewer);
 				return assistant;
 			}
 
@@ -316,48 +325,87 @@ public class SpellcheckableMessageArea extends Composite {
 		getTextWidget().addDisposeListener(new DisposeListener() {
 			public void widgetDisposed(DisposeEvent disposeEvent) {
 				support.uninstall();
+				Activator.getDefault().getPreferenceStore().removePropertyChangeListener(propertyChangeListener);
 			}
 		});
 	}
 
+	private void computeBrokenBidiPlatformTextWidth(int textWidth) {
+		class BidiSegmentListenerTester implements BidiSegmentListener {
+			boolean called;
+
+			public void lineGetSegments(BidiSegmentEvent event) {
+				called = true;
+			}
+		}
+		BidiSegmentListenerTester tester = new BidiSegmentListenerTester();
+		StyledText textWidget = getTextWidget();
+		textWidget.addBidiSegmentListener(tester);
+		textWidget.setText(" "); //$NON-NLS-1$
+		textWidget.computeSize(SWT.DEFAULT, SWT.DEFAULT);
+		textWidget.removeBidiSegmentListener(tester);
+
+		brokenBidiPlatformTextWidth = tester.called ? -1 : textWidth;
+	}
+
 	private boolean isEditable(ISourceViewer viewer) {
 		return viewer != null && viewer.getTextWidget().getEditable();
 	}
 
 	private void configureHardWrap() {
 		if (shouldHardWrap()) {
-			if (hardWrapModifyListener == null) {
+			if (hardWrapSegmentListener == null) {
 				final StyledText textWidget = getTextWidget();
-
-				hardWrapModifyListener = new ModifyListener() {
-
-					private boolean active = true;
-
-					public void modifyText(ModifyEvent e) {
-						if (!active)
-							return;
-						String lineDelimiter = textWidget.getLineDelimiter();
-						List<WrapEdit> wrapEdits = calculateWrapEdits(
-								textWidget.getText(), MAX_LINE_WIDTH,
-								lineDelimiter);
-						// Prevent infinite loop because replaceTextRange causes a ModifyEvent
-						active = false;
-						textWidget.setRedraw(false);
-						try {
-							for (WrapEdit wrapEdit : wrapEdits)
-								textWidget.replaceTextRange(wrapEdit.getStart(), wrapEdit.getLength(),
-										wrapEdit.getReplacement());
-						} finally {
-							textWidget.setRedraw(true);
-							active = true;
+				hardWrapSegmentListener = new BidiSegmentListener() {
+					public void lineGetSegments(BidiSegmentEvent e) {
+						int[] segments = calculateWrapOffsets(e.lineText, MAX_LINE_WIDTH);
+						if (segments != null) {
+							char[] segmentsChars = new char[segments.length];
+							Arrays.fill(segmentsChars, '\n');
+							e.segments = segments;
+							e.segmentsChars = segmentsChars;
 						}
 					}
 				};
-				textWidget.addModifyListener(hardWrapModifyListener);
+				textWidget.addBidiSegmentListener(hardWrapSegmentListener);
+				textWidget.setText(textWidget.getText()); // XXX: workaround for https://bugs.eclipse.org/384886
+
+				if (brokenBidiPlatformTextWidth != -1) {
+					Layout restrictedWidthLayout = new Layout() {
+						@Override
+						protected Point computeSize(Composite composite,
+								int wHint, int hHint, boolean flushCache) {
+							Point size = SpellcheckableMessageArea.this
+									.getSize();
+							Rectangle trim = SpellcheckableMessageArea.this
+									.computeTrim(0, 0, 0, 0);
+							size.x -= trim.width;
+							size.y -= trim.height;
+							if (size.x > brokenBidiPlatformTextWidth)
+								size.x = brokenBidiPlatformTextWidth;
+							return size;
+						}
+
+						@Override
+						protected void layout(Composite composite,
+								boolean flushCache) {
+							Point size = computeSize(composite, SWT.DEFAULT,
+									SWT.DEFAULT, flushCache);
+							textWidget.setBounds(0, 0, size.x, size.y);
+						}
+					};
+					setLayout(restrictedWidthLayout);
+				}
 			}
-		} else if (hardWrapModifyListener != null) {
-			getTextWidget().removeModifyListener(hardWrapModifyListener);
-			hardWrapModifyListener = null;
+
+		} else if (hardWrapSegmentListener != null) {
+			StyledText textWidget = getTextWidget();
+			textWidget.removeBidiSegmentListener(hardWrapSegmentListener);
+			textWidget.setText(textWidget.getText()); // XXX: workaround for https://bugs.eclipse.org/384886
+			hardWrapSegmentListener = null;
+
+			if (brokenBidiPlatformTextWidth != -1)
+				setLayout(new FillLayout());
 		}
 	}
 
@@ -488,10 +536,14 @@ public class SpellcheckableMessageArea extends Composite {
 			private IHandlerActivation selectAllHandlerActivation;
 			private IHandlerActivation undoHandlerActivation;
 			private IHandlerActivation redoHandlerActivation;
+			private IHandlerActivation quickFixHandlerActivation;
+			private IHandlerActivation contentAssistHandlerActivation;
 
 			public void focusGained(FocusEvent e) {
-				IHandlerService service = (IHandlerService) PlatformUI
-						.getWorkbench().getService(IHandlerService.class);
+				IHandlerService service = getHandlerService();
+				if (service == null)
+					return;
+
 				if (cutAction != null) {
 					cutAction.update();
 					cutHandlerActivation = service.activateHandler(
@@ -524,10 +576,22 @@ public class SpellcheckableMessageArea extends Composite {
 							IWorkbenchCommandConstants.EDIT_REDO,
 							new ActionHandler(redoAction),
 							new ActiveShellExpression(getParent().getShell()));
+				if (quickFixActionHandler != null)
+					quickFixHandlerActivation = getHandlerService().activateHandler(
+							quickFixActionHandler.getAction().getActionDefinitionId(),
+							quickFixActionHandler,
+							new ActiveShellExpression(getParent().getShell()));
+				if (contentAssistActionHandler != null)
+					contentAssistHandlerActivation = getHandlerService().activateHandler(
+							contentAssistActionHandler.getAction().getActionDefinitionId(),
+							contentAssistActionHandler,
+							new ActiveShellExpression(getParent().getShell()));
 			}
 
 			public void focusLost(FocusEvent e) {
-				IHandlerService service = (IHandlerService) PlatformUI.getWorkbench().getService(IHandlerService.class);
+				IHandlerService service = getHandlerService();
+				if (service == null)
+					return;
 
 				if (cutHandlerActivation != null)
 					service.deactivateHandler(cutHandlerActivation);
@@ -546,6 +610,12 @@ public class SpellcheckableMessageArea extends Composite {
 
 				if (redoHandlerActivation != null)
 					service.deactivateHandler(redoHandlerActivation);
+
+				if (quickFixHandlerActivation != null)
+					service.deactivateHandler(quickFixHandlerActivation);
+
+				if (contentAssistHandlerActivation != null)
+					service.deactivateHandler(contentAssistHandlerActivation);
 			}
 
 		});
@@ -691,24 +761,6 @@ public class SpellcheckableMessageArea extends Composite {
 		return sourceViewer.getTextWidget();
 	}
 
-	private IHandlerActivation installQuickFixActionHandler() {
-		ActionHandler handler = createQuickFixActionHandler(sourceViewer);
-		return addHandler(handler);
-	}
-
-	private IHandlerActivation installContentAssistActionHandler() {
-		ActionHandler handler = createContentAssistActionHandler(sourceViewer);
-		return addHandler(handler);
-	}
-
-	private IHandlerActivation addHandler(ActionHandler handler) {
-		ActiveShellExpression expression = new ActiveShellExpression(
-				sourceViewer.getTextWidget().getShell());
-		return getHandlerService().activateHandler(
-				handler.getAction().getActionDefinitionId(), handler,
-				expression);
-	}
-
 	private ActionHandler createQuickFixActionHandler(
 			final ITextOperationTarget textOperationTarget) {
 		Action quickFixAction = new Action() {
@@ -739,20 +791,41 @@ public class SpellcheckableMessageArea extends Composite {
 	}
 
 	/**
-	 * Return the commit message, converting platform-specific line endings.
+	 * Returns the commit message, converting platform-specific line endings to '\n'
+	 * and hard-wrapping lines if necessary.
 	 *
 	 * @return commit message
 	 */
 	public String getCommitMessage() {
 		String text = getText();
-		return Utils.normalizeLineEndings(text);
+		text = Utils.normalizeLineEndings(text);
+		if (shouldHardWrap())
+			text = hardWrap(text);
+		return text;
 	}
 
 	/**
-	 * Reconfigure this widget if a preference has changed.
+	 * Hard-wraps the given text.
+	 *
+	 * @param text the text to wrap, must use '\n' as line delimiter
+	 * @return the wrapped text
 	 */
-	public void reconfigure() {
-		configureHardWrap();
+	public static String hardWrap(String text) {
+		int[] wrapOffsets = calculateWrapOffsets(text, MAX_LINE_WIDTH);
+		if (wrapOffsets != null) {
+			StringBuilder builder = new StringBuilder(text.length() + wrapOffsets.length);
+			int prev = 0;
+			for (int cur : wrapOffsets) {
+				builder.append(text.substring(prev, cur));
+				for (int j = cur; j > prev && builder.charAt(builder.length() - 1) == ' '; j--)
+					builder.deleteCharAt(builder.length() - 1);
+				builder.append('\n');
+				prev = cur;
+			}
+			builder.append(text.substring(prev));
+			return builder.toString();
+		}
+		return text;
 	}
 
 	/**
@@ -814,124 +887,62 @@ public class SpellcheckableMessageArea extends Composite {
 	}
 
 	/**
-	 * Calculate a list of {@link WrapEdit} which can be applied to the text to
-	 * get a new text that is wrapped at word boundaries. Existing line breaks
-	 * are left alone (text is not reflowed).
+	 * Calculates wrap offsets for the given line, so that resulting lines are
+	 * no longer than <code>maxLineLength</code> if possible.
 	 *
-	 * @param text
-	 *            the text to calculate the wrap edits for
+	 * @param line
+	 *            the line to wrap (can contain '\n', but no other line delimiters)
 	 * @param maxLineLength
 	 *            the maximum line length
-	 * @param lineDelimiter
-	 *            line delimiter used in text and for wrapping
-	 * @return a list of {@link WrapEdit} objects which specify how the text
-	 *         should be edited to obtain the wrapped text. Offsets of later
-	 *         edits are already adjusted for the fact that wrapping a line may
-	 *         shift the text backwards. So the list can just be iterated and
-	 *         each edit applied in order.
+	 * @return an array of offsets where hard-wraps should be inserted, or
+	 *         <code>null</code> if the line does not need to be wrapped
 	 */
-	public static List<WrapEdit> calculateWrapEdits(final String text, final int maxLineLength, final String lineDelimiter) {
-		List<WrapEdit> wrapEdits = new LinkedList<WrapEdit>();
+	public static int[] calculateWrapOffsets(final String line, final int maxLineLength) {
+		if (line.length() == 0)
+			return null;
 
-		final int lineDelimiterLength = lineDelimiter.length();
-		final int spaceLength = 1;
-
-		int offset = 0;
-		boolean lastChunkWasWrapped = false;
-		int lastChunkWrappedOffset = 0;
-
-		String[] chunks = text.split(lineDelimiter, -1);
-		for (int chunkIndex = 0; chunkIndex < chunks.length; chunkIndex++) {
-			String chunk = chunks[chunkIndex];
-
-			String[] words = chunk.split(" ", -1); //$NON-NLS-1$
-			int lineLength = 0;
-
-			for (int wordIndex = 0; wordIndex < words.length; wordIndex++) {
-				String word = words[wordIndex];
-				int wordLength = word.length();
-				boolean adjustForSpace = wordIndex != 0;
-
-				if (wordIndex == 0 && lastChunkWasWrapped) {
-					if (wordLength != 0 && !STARTS_WITH_SYMBOL.matcher(word).matches()) {
-						wrapEdits.add(new WrapEdit(offset - lineDelimiterLength, lineDelimiterLength, " ")); //$NON-NLS-1$
-						/* adjust for join edit above */
-						offset -= lineDelimiterLength;
-						adjustForSpace = true;
-						lineLength = offset - lastChunkWrappedOffset;
+		IntList wrapOffsets = new IntList();
+		int wordStart = 0;
+		int lineStart = 0;
+		boolean lastWasSpace = true;
+		boolean onlySpaces = true;
+		for (int i = 0; i < line.length(); i++) {
+			char ch = line.charAt(i);
+			if (ch == ' ') {
+				lastWasSpace = true;
+			} else if (ch == '\n') {
+				lineStart = i + 1;
+				wordStart = i + 1;
+				lastWasSpace = true;
+				onlySpaces = true;
+			} else { // a word character
+				if (lastWasSpace) {
+					lastWasSpace = false;
+					if (!onlySpaces) { // don't break line with <spaces><veryLongWord>
+						wordStart = i;
 					}
-					lastChunkWasWrapped = false;
+				} else {
+					onlySpaces = false;
 				}
-
-				int newLineLength = lineLength + spaceLength + wordLength;
-				if (newLineLength > maxLineLength) {
-					/* don't break before a single long word */
-					if (lineLength != 0) {
-						wrapEdits.add(new WrapEdit(offset, spaceLength, lineDelimiter));
-						/* adjust for the shifting of text after the edit is applied */
-						offset += lineDelimiterLength;
-						adjustForSpace = false;
-						lastChunkWasWrapped = true;
-						lastChunkWrappedOffset = offset;
+				if (i >= lineStart + maxLineLength) {
+					if (wordStart != lineStart) { // don't break before a single long word
+						wrapOffsets.add(wordStart);
+						lineStart = wordStart;
+						onlySpaces = true;
 					}
-					lineLength = 0;
 				}
-
-				if (adjustForSpace) {
-					offset += spaceLength;
-					lineLength += spaceLength;
-				}
-
-				offset += wordLength;
-				lineLength += wordLength;
 			}
-
-			offset += lineDelimiterLength;
 		}
 
-		return wrapEdits;
-	}
-
-	/**
-	 * Edit for replacing a text range with another text to wrap or join lines.
-	 */
-	public static class WrapEdit {
-		private final int start;
-		private final int length;
-		private final String replacement;
-
-		/**
-		 * @param start see {@link #getStart()}
-		 * @param length see {@link #getLength()}
-		 * @param replacement see {@link #getReplacement()}
-		 */
-		public WrapEdit(int start, int length, String replacement) {
-			this.start = start;
-			this.length = length;
-			this.replacement = replacement;
-		}
-
-		/**
-		 * @return character offset of where the edit should be applied on the
-		 *         text
-		 */
-		public int getStart() {
-			return start;
-		}
-
-		/**
-		 * @return number of characters which should be replaced by the
-		 *         replacement text
-		 */
-		public int getLength() {
-			return length;
-		}
-
-		/**
-		 * @return the text which replaces the edit range
-		 */
-		public String getReplacement() {
-			return replacement;
+		int size = wrapOffsets.size();
+		if (size == 0) {
+			return null;
+		} else {
+			int[] result = new int[size];
+			for (int i = 0; i < size; i++) {
+				result[i] = wrapOffsets.get(i);
+			}
+			return result;
 		}
 	}
 }
