@@ -1,5 +1,5 @@
 /*******************************************************************************
- * Copyright (C) 2011, Mathias Kinzler <mathias.kinzler@sap.com> and others.
+ * Copyright (C) 2011, 2013 Mathias Kinzler <mathias.kinzler@sap.com> and others.
  *
  * All rights reserved. This program and the accompanying materials
  * are made available under the terms of the Eclipse Public License v1.0
@@ -10,9 +10,13 @@ package org.eclipse.egit.ui.internal.history;
 
 import java.io.IOException;
 import java.lang.reflect.InvocationTargetException;
+import java.util.ArrayList;
+import java.util.List;
 import java.util.Map.Entry;
 
+import org.eclipse.core.resources.IResource;
 import org.eclipse.core.runtime.IProgressMonitor;
+import org.eclipse.egit.core.project.RepositoryMapping;
 import org.eclipse.egit.ui.Activator;
 import org.eclipse.egit.ui.UIPreferences;
 import org.eclipse.egit.ui.UIText;
@@ -35,9 +39,14 @@ import org.eclipse.jgit.lib.Constants;
 import org.eclipse.jgit.lib.ObjectId;
 import org.eclipse.jgit.lib.Ref;
 import org.eclipse.jgit.lib.Repository;
+import org.eclipse.jgit.revwalk.FollowFilter;
 import org.eclipse.jgit.revwalk.RevFlag;
 import org.eclipse.jgit.revwalk.RevSort;
 import org.eclipse.jgit.revwalk.RevWalk;
+import org.eclipse.jgit.treewalk.filter.AndTreeFilter;
+import org.eclipse.jgit.treewalk.filter.OrTreeFilter;
+import org.eclipse.jgit.treewalk.filter.PathFilter;
+import org.eclipse.jgit.treewalk.filter.TreeFilter;
 import org.eclipse.osgi.util.NLS;
 import org.eclipse.swt.SWT;
 import org.eclipse.swt.layout.GridLayout;
@@ -54,6 +63,8 @@ public class CommitSelectionDialog extends TitleAreaDialog {
 
 	private final Repository repository;
 
+	private final IResource[] filterResources;
+
 	private CommitGraphTable table;
 
 	private SWTCommitList allCommits;
@@ -67,9 +78,24 @@ public class CommitSelectionDialog extends TitleAreaDialog {
 	 * @param repository
 	 */
 	public CommitSelectionDialog(Shell parentShell, Repository repository) {
+		this(parentShell, repository, null);
+	}
+
+	/**
+	 * Create a commit selection dialog which shows only commits which changed
+	 * the given resources.
+	 *
+	 * @param parentShell
+	 * @param repository
+	 * @param filterResources
+	 *            the resources to use to filter commits, null for no filter
+	 */
+	public CommitSelectionDialog(Shell parentShell, Repository repository,
+			IResource[] filterResources) {
 		super(parentShell);
 		setShellStyle(getShellStyle() | SWT.SHELL_TRIM);
 		this.repository = repository;
+		this.filterResources = filterResources;
 	}
 
 	@Override
@@ -128,6 +154,7 @@ public class CommitSelectionDialog extends TitleAreaDialog {
 												UIText.CommitSelectionDialog_BuildingCommitListMessage,
 												IProgressMonitor.UNKNOWN);
 								SWTWalk currentWalk = new SWTWalk(repository);
+								currentWalk.setTreeFilter(createTreeFilter());
 								currentWalk
 										.sort(RevSort.COMMIT_TIME_DESC, true);
 								currentWalk.sort(RevSort.BOUNDARY, true);
@@ -220,4 +247,30 @@ public class CommitSelectionDialog extends TitleAreaDialog {
 		}
 	}
 
+	private TreeFilter createTreeFilter() {
+		if (filterResources == null)
+			return TreeFilter.ALL;
+
+		List<TreeFilter> filters = new ArrayList<TreeFilter>();
+		for (IResource resource : filterResources) {
+			RepositoryMapping mapping = RepositoryMapping.getMapping(resource);
+			if (mapping != null) {
+				String path = mapping.getRepoRelativePath(resource);
+				if (path != null && !"".equals(path)) { //$NON-NLS-1$
+					if (resource.getType() == IResource.FILE)
+						filters.add(FollowFilter.create(path));
+					else
+						filters.add(AndTreeFilter.create(
+								PathFilter.create(path), TreeFilter.ANY_DIFF));
+				}
+			}
+		}
+
+		if (filters.isEmpty())
+			return TreeFilter.ALL;
+		else if (filters.size() == 1)
+			return filters.get(0);
+		else
+			return OrTreeFilter.create(filters);
+	}
 }
