@@ -5,6 +5,8 @@
  * Copyright (C) 2010, Mathias Kinzler <mathias.kinzler@sap.com>
  * Copyright (C) 2010-2012, Matthias Sohn <matthias.sohn@sap.com>
  * Copyright (C) 2012, Daniel megert <daniel_megert@ch.ibm.com>
+ * Copyright (C) 2012, Robin Stocker <robin@nibor.org>
+ * Copyright (C) 2012, François Rey <eclipse.org_@_francois_._rey_._name>
  *
  * All rights reserved. This program and the accompanying materials
  * are made available under the terms of the Eclipse Public License v1.0
@@ -16,8 +18,10 @@ package org.eclipse.egit.ui.internal.history;
 import java.io.File;
 import java.io.IOException;
 import java.util.ArrayList;
+import java.util.HashSet;
 import java.util.List;
 import java.util.Map.Entry;
+import java.util.Set;
 
 import org.eclipse.core.resources.IResource;
 import org.eclipse.core.resources.IWorkspaceRoot;
@@ -62,6 +66,7 @@ import org.eclipse.jface.viewers.ISelectionChangedListener;
 import org.eclipse.jface.viewers.ISelectionProvider;
 import org.eclipse.jface.viewers.IStructuredSelection;
 import org.eclipse.jface.viewers.SelectionChangedEvent;
+import org.eclipse.jgit.diff.DiffEntry;
 import org.eclipse.jgit.errors.IncorrectObjectTypeException;
 import org.eclipse.jgit.errors.MissingObjectException;
 import org.eclipse.jgit.events.ListenerHandle;
@@ -75,6 +80,7 @@ import org.eclipse.jgit.lib.Repository;
 import org.eclipse.jgit.revplot.PlotCommit;
 import org.eclipse.jgit.revplot.PlotWalk;
 import org.eclipse.jgit.revwalk.FollowFilter;
+import org.eclipse.jgit.revwalk.RenameCallback;
 import org.eclipse.jgit.revwalk.RevCommit;
 import org.eclipse.jgit.revwalk.RevFlag;
 import org.eclipse.jgit.revwalk.RevObject;
@@ -94,8 +100,6 @@ import org.eclipse.swt.custom.StackLayout;
 import org.eclipse.swt.custom.StyledText;
 import org.eclipse.swt.events.DisposeEvent;
 import org.eclipse.swt.events.DisposeListener;
-import org.eclipse.swt.events.MenuDetectEvent;
-import org.eclipse.swt.events.MenuDetectListener;
 import org.eclipse.swt.events.SelectionAdapter;
 import org.eclipse.swt.events.SelectionEvent;
 import org.eclipse.swt.layout.GridData;
@@ -112,11 +116,13 @@ import org.eclipse.ui.IWorkbenchPartSite;
 import org.eclipse.ui.PlatformUI;
 import org.eclipse.ui.actions.ActionFactory.IWorkbenchAction;
 import org.eclipse.ui.dialogs.PreferencesUtil;
+import org.eclipse.ui.part.IShowInSource;
+import org.eclipse.ui.part.ShowInContext;
 import org.eclipse.ui.progress.IWorkbenchSiteProgressService;
 
 /** Graphical commit history viewer. */
 public class GitHistoryPage extends HistoryPage implements RefsChangedListener,
-		ISchedulingRule, TableLoader {
+		ISchedulingRule, TableLoader, IShowInSource {
 
 	private static final int INITIAL_ITEM = -1;
 
@@ -644,6 +650,9 @@ public class GitHistoryPage extends HistoryPage implements RefsChangedListener,
 
 	private boolean currentFollowRenames;
 
+	/** Tracks the file names that are to be highlighted in the diff file viewer */
+	private Set<String> fileViewerInterestingPaths;
+
 	// react on changes to the relative date preference
 	private final IPropertyChangeListener listener = new IPropertyChangeListener() {
 		public void propertyChange(PropertyChangeEvent event) {
@@ -916,13 +925,6 @@ public class GitHistoryPage extends HistoryPage implements RefsChangedListener,
 		actions = new GitHistoryPageActions(this);
 		setupToolBar();
 		setupViewMenu();
-
-		graph.getControl().addMenuDetectListener(new MenuDetectListener() {
-			public void menuDetected(MenuDetectEvent e) {
-				popupMgr.add(actions.showFilesAction);
-				popupMgr.add(actions.showCommentAction);
-			}
-		});
 	}
 
 	private void setupToolBar() {
@@ -1474,6 +1476,8 @@ public class GitHistoryPage extends HistoryPage implements RefsChangedListener,
 					graph.setInput(highlightFlag, list, asArray, input, true);
 					if (toSelect != null)
 						graph.selectCommit(toSelect);
+					if (getFollowRenames())
+						updateInterestingPathsOfFileViewer();
 					if (trace)
 						GitTraceLocation.getTrace().trace(
 								GitTraceLocation.HISTORYVIEW.getLocation(),
@@ -1493,6 +1497,10 @@ public class GitHistoryPage extends HistoryPage implements RefsChangedListener,
 					GitTraceLocation.HISTORYVIEW.getLocation());
 	}
 
+	private void updateInterestingPathsOfFileViewer() {
+		fileViewer.setInterestingPaths(fileViewerInterestingPaths);
+	}
+
 	private void setWarningText(String warning) {
 		if (warningComposite == null || warningComposite.isDisposed())
 			return;
@@ -1620,7 +1628,7 @@ public class GitHistoryPage extends HistoryPage implements RefsChangedListener,
 					continue;
 				if (db != map.getRepository())
 					throw new IllegalStateException(
-							UIText.AbstractHistoryCommanndHandler_NoUniqueRepository);
+							UIText.RepositoryAction_multiRepoSelection);
 
 				if (showAllFilter == ShowFilter.SHOWALLFOLDER) {
 					final String path;
@@ -1744,6 +1752,7 @@ public class GitHistoryPage extends HistoryPage implements RefsChangedListener,
 	private TreeWalk setupFileViewer(RevWalk walk, Repository db, List<FilterPath> paths) {
 		final TreeWalk fileWalker = createFileWalker(walk, db, paths);
 		fileViewer.setTreeWalk(db, fileWalker);
+		fileViewer.setInterestingPaths(fileViewerInterestingPaths);
 		fileViewer.refresh();
 		fileViewer.addSelectionChangedListener(commentViewer);
 		return fileWalker;
@@ -1762,6 +1771,7 @@ public class GitHistoryPage extends HistoryPage implements RefsChangedListener,
 			for (FilterPath filterPath : paths)
 				selectedPaths.add(filterPath.getPath());
 
+			fileViewerInterestingPaths = new HashSet<String>(selectedPaths);
 			TreeFilter followFilter = createFollowFilterFor(selectedPaths);
 			walk.setTreeFilter(followFilter);
 		} else if (paths.size() > 0) {
@@ -1772,9 +1782,11 @@ public class GitHistoryPage extends HistoryPage implements RefsChangedListener,
 
 			walk.setTreeFilter(AndTreeFilter.create(PathFilterGroup
 					.createFromStrings(stringPaths), TreeFilter.ANY_DIFF));
+			fileViewerInterestingPaths = new HashSet<String>(stringPaths);
 		} else {
 			pathFilters = null;
 			walk.setTreeFilter(TreeFilter.ALL);
+			fileViewerInterestingPaths = null;
 		}
 		return fileWalker;
 	}
@@ -1791,7 +1803,7 @@ public class GitHistoryPage extends HistoryPage implements RefsChangedListener,
 
 		List<TreeFilter> followFilters = new ArrayList<TreeFilter>(paths.size());
 		for (String path : paths)
-			followFilters.add(FollowFilter.create(path));
+			followFilters.add(createFollowFilter(path));
 
 		if (followFilters.size() == 1)
 			return followFilters.get(0);
@@ -1799,6 +1811,20 @@ public class GitHistoryPage extends HistoryPage implements RefsChangedListener,
 		return OrTreeFilter.create(followFilters);
 	}
 
+	private FollowFilter createFollowFilter(String path) {
+		FollowFilter followFilter = FollowFilter.create(path);
+		followFilter.setRenameCallback(new RenameCallback() {
+			@Override
+			public void renamed(DiffEntry entry) {
+				if (fileViewerInterestingPaths != null) {
+					fileViewerInterestingPaths.add(entry.getOldPath());
+					fileViewerInterestingPaths.add(entry.getNewPath());
+				}
+			}
+		});
+		return followFilter;
+	}
+
 	/**
 	 * @return Returns <code>true</code> if <b>all</b> filterpaths refer to plain files,
 	 * 			or if the list is empty.
@@ -1948,4 +1974,11 @@ public class GitHistoryPage extends HistoryPage implements RefsChangedListener,
 	public boolean isConflicting(ISchedulingRule rule) {
 		return this == rule;
 	}
+
+	public ShowInContext getShowInContext() {
+		if (fileViewer != null && fileViewer.getControl().isFocusControl())
+			return fileViewer.getShowInContext();
+		else
+			return null;
+	}
 }
