@@ -69,9 +69,9 @@ public class MergeHandler extends AbstractHistoryCommandHandler {
 					HandlerUtil.getActiveShellChecked(event), nodes,
 					UIText.MergeHandler_SelectBranchTitle,
 					UIText.MergeHandler_SelectBranchMessage, SWT.SINGLE);
-			if (dlg.open() == Window.OK) {
+			if (dlg.open() == Window.OK)
 				refName = dlg.getSelectedNode().getObject().getName();
-			} else
+			else
 				return null;
 		}
 		String jobname = NLS.bind(UIText.MergeAction_JobNameMerge, refName);
@@ -93,7 +93,7 @@ public class MergeHandler extends AbstractHistoryCommandHandler {
 			@Override
 			public void done(IJobChangeEvent cevent) {
 				IStatus result = cevent.getJob().getResult();
-				if (result.getSeverity() == IStatus.CANCEL) {
+				if (result.getSeverity() == IStatus.CANCEL)
 					Display.getDefault().asyncExec(new Runnable() {
 						public void run() {
 							// don't use getShell(event) here since
@@ -108,19 +108,18 @@ public class MergeHandler extends AbstractHistoryCommandHandler {
 											UIText.MergeAction_MergeCanceledMessage);
 						}
 					});
-				} else if (!result.isOK()) {
+				else if (!result.isOK())
 					Activator.handleError(result.getMessage(), result
 							.getException(), true);
-				} else {
+				else
 					Display.getDefault().asyncExec(new Runnable() {
 						public void run() {
 							Shell shell = PlatformUI.getWorkbench()
 									.getActiveWorkbenchWindow().getShell();
-							new MergeResultDialog(shell, repository, op
+							MergeResultDialog.getDialog(shell, repository, op
 									.getResult()).open();
 						}
 					});
-				}
 			}
 		});
 		job.schedule();
@@ -146,9 +145,8 @@ public class MergeHandler extends AbstractHistoryCommandHandler {
 			ex = e;
 		}
 
-		if (message != null) {
+		if (message != null)
 			Activator.handleError(UIText.MergeAction_CannotMerge, ex, true);
-		}
 		return (message == null);
 	}
 
