@@ -1,5 +1,5 @@
 /*******************************************************************************
- * Copyright (C) 2010, 2011 Mathias Kinzler <mathias.kinzler@sap.com> and others.
+ * Copyright (C) 2010, 2013 Mathias Kinzler <mathias.kinzler@sap.com> and others.
  *
  * All rights reserved. This program and the accompanying materials
  * are made available under the terms of the Eclipse Public License v1.0
@@ -28,12 +28,14 @@ import org.eclipse.core.runtime.CoreException;
 import org.eclipse.core.runtime.IPath;
 import org.eclipse.core.runtime.IProgressMonitor;
 import org.eclipse.core.runtime.Path;
+import org.eclipse.egit.core.internal.CompareCoreUtils;
 import org.eclipse.egit.core.internal.storage.GitFileRevision;
 import org.eclipse.egit.core.project.RepositoryMapping;
 import org.eclipse.egit.ui.Activator;
 import org.eclipse.egit.ui.UIText;
 import org.eclipse.egit.ui.internal.CompareUtils;
 import org.eclipse.egit.ui.internal.FileEditableRevision;
+import org.eclipse.egit.ui.internal.FileRevisionTypedElement;
 import org.eclipse.egit.ui.internal.LocalFileRevision;
 import org.eclipse.egit.ui.internal.GitCompareFileRevisionEditorInput.EmptyTypedElement;
 import org.eclipse.jface.operation.IRunnableContext;
@@ -113,7 +115,7 @@ public class GitMergeEditorInput extends CompareEditorInput {
 				if (repo != null && repo != map.getRepository())
 					throw new InvocationTargetException(
 							new IllegalStateException(
-									UIText.AbstractHistoryCommanndHandler_NoUniqueRepository));
+									UIText.RepositoryAction_multiRepoSelection));
 				filterPaths.add(map.getRepoRelativePath(resource));
 				repo = map.getRepository();
 			}
@@ -121,7 +123,7 @@ public class GitMergeEditorInput extends CompareEditorInput {
 			if (repo == null)
 				throw new InvocationTargetException(
 						new IllegalStateException(
-								UIText.AbstractHistoryCommanndHandler_NoUniqueRepository));
+								UIText.RepositoryAction_multiRepoSelection));
 
 			if (monitor.isCanceled())
 				throw new InterruptedException();
@@ -193,17 +195,17 @@ public class GitMergeEditorInput extends CompareEditorInput {
 			// set the labels
 			CompareConfiguration config = getCompareConfiguration();
 			config.setRightLabel(NLS.bind(LABELPATTERN, rightCommit
-					.getShortMessage(), rightCommit.name()));
+					.getShortMessage(), CompareUtils.truncatedRevision(rightCommit.name())));
 
 			if (!useWorkspace)
 				config.setLeftLabel(NLS.bind(LABELPATTERN, headCommit
-						.getShortMessage(), headCommit.name()));
+						.getShortMessage(), CompareUtils.truncatedRevision(headCommit.name())));
 			else
 				config.setLeftLabel(UIText.GitMergeEditorInput_WorkspaceHeader);
 
 			if (ancestorCommit != null)
 				config.setAncestorLabel(NLS.bind(LABELPATTERN, ancestorCommit
-						.getShortMessage(), ancestorCommit.name()));
+						.getShortMessage(), CompareUtils.truncatedRevision(ancestorCommit.name())));
 
 			// set title and icon
 			setTitle(NLS.bind(UIText.GitMergeEditorInput_MergeEditorTitle,
@@ -310,10 +312,13 @@ public class GitMergeEditorInput extends CompareEditorInput {
 					continue;
 
 				ITypedElement right;
-				if (conflicting)
-					right = CompareUtils.getFileRevisionTypedElement(gitPath,
-							rightCommit, repository);
-				else
+				if (conflicting) {
+					GitFileRevision revision = GitFileRevision.inIndex(
+							repository, gitPath, DirCacheEntry.STAGE_3);
+					String encoding = CompareCoreUtils.getResourceEncoding(
+							repository, gitPath);
+					right = new FileRevisionTypedElement(revision, encoding);
+				} else
 					right = CompareUtils.getFileRevisionTypedElement(gitPath,
 							headCommit, repository);
 
@@ -335,8 +340,8 @@ public class GitMergeEditorInput extends CompareEditorInput {
 				if (!conflicting || useWorkspace)
 					rev = new LocalFileRevision(file);
 				else
-					rev = GitFileRevision.inCommit(repository, headCommit,
-							gitPath, null);
+					rev = GitFileRevision.inIndex(repository, gitPath,
+							DirCacheEntry.STAGE_2);
 
 				IRunnableContext runnableContext = getContainer();
 				if (runnableContext == null)
