@@ -165,9 +165,8 @@ public class ConfigurationEditorComponent {
 	public void restore() throws IOException {
 		if (changeablePath) {
 			try {
-				IEclipsePreferences node = new InstanceScope()
-						.getNode(org.eclipse.egit.core.Activator
-								.getPluginId());
+				IEclipsePreferences node = InstanceScope.INSTANCE
+						.getNode(org.eclipse.egit.core.Activator.getPluginId());
 				node.remove(GitCorePreferences.core_gitPrefix);
 				node.flush();
 				// Create a temporary FS instance to compute the Git prefix
@@ -243,7 +242,7 @@ public class ConfigurationEditorComponent {
 							}
 							location.setText(file);
 							try {
-								IEclipsePreferences node = new InstanceScope()
+								IEclipsePreferences node = InstanceScope.INSTANCE
 										.getNode(org.eclipse.egit.core.Activator
 												.getPluginId());
 								node.put(GitCorePreferences.core_gitPrefix,
