@@ -8,6 +8,7 @@
  * Contributors:
  *   Chris Aniszczyk <caniszczyk@gmail.com> - initial implementation
  *   EclipseSource - Filtered Viewer
+ *   Robin Stocker <robin@nibor.org> - Show In support
  *******************************************************************************/
 package org.eclipse.egit.ui.internal.reflog;
 
@@ -87,13 +88,15 @@ import org.eclipse.ui.forms.events.HyperlinkEvent;
 import org.eclipse.ui.forms.widgets.Form;
 import org.eclipse.ui.forms.widgets.FormToolkit;
 import org.eclipse.ui.forms.widgets.ImageHyperlink;
+import org.eclipse.ui.part.IShowInTarget;
+import org.eclipse.ui.part.ShowInContext;
 import org.eclipse.ui.part.ViewPart;
 
 /**
  * A view that shows reflog entries. The View includes a quick filter that
  * searches on both the commit hashes and commit messages.
  */
-public class ReflogView extends ViewPart implements RefsChangedListener {
+public class ReflogView extends ViewPart implements RefsChangedListener, IShowInTarget {
 
 	/**
 	 * View id
@@ -470,12 +473,27 @@ public class ReflogView extends ViewPart implements RefsChangedListener {
 		return null;
 	}
 
+	public boolean show(ShowInContext context) {
+		ISelection selection = context.getSelection();
+		if (selection instanceof IStructuredSelection) {
+			IStructuredSelection structuredSelection = (IStructuredSelection) selection;
+			for (Object element : structuredSelection.toList()) {
+				if (element instanceof RepositoryTreeNode) {
+					RepositoryTreeNode node = (RepositoryTreeNode) element;
+					showReflogFor(node.getRepository());
+					return true;
+				}
+			}
+		}
+		return false;
+	}
+
 	/**
 	 * Defines the repository for the reflog to show.
 	 *
 	 * @param repository
 	 */
-	public void showReflogFor(Repository repository) {
+	private void showReflogFor(Repository repository) {
 		showReflogFor(repository, Constants.HEAD);
 	}
 
