@@ -25,14 +25,12 @@ import org.eclipse.core.commands.ExecutionEvent;
 import org.eclipse.core.commands.ExecutionException;
 import org.eclipse.core.expressions.IEvaluationContext;
 import org.eclipse.core.runtime.IPath;
-import org.eclipse.core.runtime.Path;
 import org.eclipse.egit.core.RepositoryUtil;
 import org.eclipse.egit.ui.Activator;
 import org.eclipse.egit.ui.internal.repository.RepositoriesView;
 import org.eclipse.egit.ui.internal.repository.tree.FileNode;
 import org.eclipse.egit.ui.internal.repository.tree.FolderNode;
 import org.eclipse.egit.ui.internal.repository.tree.RepositoryTreeNode;
-import org.eclipse.egit.ui.internal.repository.tree.RepositoryTreeNodeType;
 import org.eclipse.egit.ui.internal.repository.tree.WorkingDirNode;
 import org.eclipse.jface.viewers.ISelection;
 import org.eclipse.jface.viewers.IStructuredSelection;
@@ -210,22 +208,10 @@ abstract class RepositoriesViewCommandHandler<T> extends AbstractHandler {
 	protected Collection<IPath> getSelectedFileAndFolderPaths(ExecutionEvent event) throws ExecutionException {
 		Collection<IPath> paths = new ArrayList<IPath>();
 		for (Object selectedNode : getSelectedNodes(event)) {
-			IPath path = getPath((RepositoryTreeNode) selectedNode);
+			RepositoryTreeNode treeNode = (RepositoryTreeNode) selectedNode;
+			IPath path = treeNode.getPath();
 			paths.add(path);
 		}
 		return paths;
 	}
-
-	protected IPath getPath(RepositoryTreeNode node) {
-		Repository repository = node.getRepository();
-		IPath path;
-		if (node.getType().equals(RepositoryTreeNodeType.FOLDER))
-			path = new Path(((FolderNode) node).getObject()
-					.getAbsolutePath());
-		else if (node.getType().equals(RepositoryTreeNodeType.FILE))
-			path = new Path(((FileNode) node).getObject().getAbsolutePath());
-		else
-			path = new Path(repository.getWorkTree().getAbsolutePath());
-		return path;
-	}
 }
