@@ -22,6 +22,7 @@ import org.eclipse.core.runtime.CoreException;
 import org.eclipse.core.runtime.IAdaptable;
 import org.eclipse.core.runtime.IPath;
 import org.eclipse.core.runtime.Path;
+import org.eclipse.egit.ui.internal.decorators.IDecoratableResource;
 import org.eclipse.egit.ui.internal.decorators.IProblemDecoratable;
 import org.eclipse.jgit.lib.Repository;
 
@@ -29,7 +30,7 @@ import org.eclipse.jgit.lib.Repository;
 /**
  * A staged/unstaged entry in the table
  */
-public class StagingEntry implements IAdaptable, IProblemDecoratable {
+public class StagingEntry implements IAdaptable, IProblemDecoratable, IDecoratableResource {
 	/**
 	 * State of the node
 	 */
@@ -53,7 +54,7 @@ public class StagingEntry implements IAdaptable, IProblemDecoratable {
 		PARTIALLY_MODIFIED(EnumSet.of(Action.REPLACE_WITH_FILE_IN_GIT_INDEX, Action.REPLACE_WITH_HEAD_REVISION, Action.STAGE)),
 
 		/** not ignored, and not in the index */
-		UNTRACKED(EnumSet.of(Action.STAGE, Action.DELETE)),
+		UNTRACKED(EnumSet.of(Action.STAGE, Action.DELETE, Action.IGNORE)),
 
 		/** in conflict */
 		CONFLICTING(EnumSet.of(Action.REPLACE_WITH_FILE_IN_GIT_INDEX, Action.REPLACE_WITH_HEAD_REVISION,
@@ -82,6 +83,7 @@ public class StagingEntry implements IAdaptable, IProblemDecoratable {
 		STAGE,
 		UNSTAGE,
 		DELETE,
+		IGNORE,
 		LAUNCH_MERGE_TOOL,
 	}
 
@@ -151,8 +153,6 @@ public class StagingEntry implements IAdaptable, IProblemDecoratable {
 		IPath absolutePath = getLocation();
 		IWorkspaceRoot root = ResourcesPlugin.getWorkspace().getRoot();
 		IFile resource = root.getFileForLocation(absolutePath);
-		if (resource == null)
-			resource = root.getFile(absolutePath);
 		return resource;
 	}
 
@@ -165,8 +165,7 @@ public class StagingEntry implements IAdaptable, IProblemDecoratable {
 	}
 
 	public int getProblemSeverity() {
-		IWorkspaceRoot root = ResourcesPlugin.getWorkspace().getRoot();
-		IFile file = root.getFileForLocation(getLocation());
+		IFile file = getFile();
 		if (file == null)
 			return SEVERITY_NONE;
 
@@ -178,12 +177,69 @@ public class StagingEntry implements IAdaptable, IProblemDecoratable {
 	}
 
 	public Object getAdapter(Class adapter) {
-		if (adapter == IResource.class) {
+		if (adapter == IResource.class)
 			return getFile();
-		}
+		else if (adapter == IPath.class)
+			return getLocation();
 		return null;
 	}
 
+	public int getType() {
+		return IResource.FILE;
+	}
+
+	public String getName() {
+		// Not used in StagingViewLabelProvider
+		return null;
+	}
+
+	public String getRepositoryName() {
+		return null;
+	}
+
+	public String getBranch() {
+		return null;
+	}
+
+	public String getBranchStatus() {
+		return null;
+	}
+
+	public boolean isTracked() {
+		return state != State.UNTRACKED;
+	}
+
+	public boolean isIgnored() {
+		return false;
+	}
+
+	public boolean isDirty() {
+		return state == State.MODIFIED || state == State.PARTIALLY_MODIFIED;
+	}
+
+	public Staged staged() {
+		switch (state) {
+		case ADDED:
+			return Staged.ADDED;
+		case CHANGED:
+			return Staged.MODIFIED;
+		case REMOVED:
+			return Staged.REMOVED;
+		case MISSING:
+			return Staged.REMOVED;
+		default:
+			return Staged.NOT_STAGED;
+		}
+	}
+
+	public boolean hasConflicts() {
+		return state == State.CONFLICTING;
+	}
+
+	public boolean isAssumeValid() {
+		return false;
+	}
+
 	@Override
 	public int hashCode() {
 		final int prime = 31;
