@@ -1,5 +1,5 @@
 /*******************************************************************************
- * Copyright (C) 2010, Dariusz Luksza <dariusz@luksza.org>
+ * Copyright (C) 2010, 2013 Dariusz Luksza <dariusz@luksza.org> and others.
  *
  * All rights reserved. This program and the accompanying materials
  * are made available under the terms of the Eclipse Public License v1.0
@@ -8,17 +8,15 @@
  *******************************************************************************/
 package org.eclipse.egit.ui.internal.synchronize.model;
 
-import java.util.ArrayList;
-import java.util.HashMap;
-import java.util.List;
 import java.util.Map;
-import java.util.Map.Entry;
 
 import org.eclipse.compare.structuremergeviewer.Differencer;
 import org.eclipse.core.runtime.IPath;
 import org.eclipse.core.runtime.Path;
 import org.eclipse.egit.core.synchronize.GitCommitsModelCache.Change;
 import org.eclipse.egit.ui.UIText;
+import org.eclipse.egit.ui.internal.synchronize.model.TreeBuilder.FileModelFactory;
+import org.eclipse.egit.ui.internal.synchronize.model.TreeBuilder.TreeModelFactory;
 import org.eclipse.jgit.lib.Repository;
 
 /**
@@ -28,45 +26,9 @@ public class GitModelCache extends GitModelObjectContainer {
 
 	private final Path location;
 
-	private final FileModelFactory fileFactory;
-
-	private final Map<String, GitModelCacheTree> cacheTreeMap;
-
 	private final Repository repo;
 
-	private final Map<String, Change> cache;
-
-	/**
-	 * This interface enables creating proper instance of {@link GitModelBlob}
-	 * for cached and working files. In case of working files the left side
-	 * content of Compare View is loaded from local hard drive.
-	 */
-	protected interface FileModelFactory {
-		/**
-		 * Creates proper instance of {@link GitModelBlob} for cache and working
-		 * tree model representation
-		 *
-		 * @param objParent
-		 *            parent object
-		 * @param repo
-		 *            repository associated with file that will be created
-		 * @param change
-		 *            change associated with file that will be created
-		 * @param fullPath
-		 *            absolute path
-		 * @return instance of {@link GitModelBlob}
-		 */
-		GitModelBlob createFileModel(GitModelObjectContainer objParent,
-				Repository repo, Change change, IPath fullPath);
-
-		/**
-		 * Distinguish working tree from changed/staged tree
-		 *
-		 * @return {@code true} when this tree is working tree, {@code false}
-		 *         when it is a cached tree
-		 */
-		boolean isWorkingTree();
-	}
+	private GitModelObject[] children;
 
 	/**
 	 * Constructs model node that represents current status of Git cache.
@@ -100,19 +62,27 @@ public class GitModelCache extends GitModelObjectContainer {
 	 *            parent object
 	 * @param repo
 	 *            repository associated with this object
-	 * @param cache
+	 * @param changes
 	 *            list of changes associated with this object
 	 * @param fileFactory
 	 *            leaf instance factory
 	 */
-	protected GitModelCache(GitModelRepository parent, Repository repo, Map<String, Change> cache,
-			FileModelFactory fileFactory) {
+	protected GitModelCache(GitModelRepository parent, final Repository repo,
+			Map<String, Change> changes, final FileModelFactory fileFactory) {
 		super(parent);
 		this.repo = repo;
-		this.cache = cache;
-		this.fileFactory = fileFactory;
-		cacheTreeMap = new HashMap<String, GitModelCacheTree>();
-		location = new Path(repo.getWorkTree().toString());
+		this.location = new Path(repo.getWorkTree().toString());
+
+		this.children = TreeBuilder.build(this, repo, changes, fileFactory,
+				new TreeModelFactory() {
+					public GitModelTree createTreeModel(
+							GitModelObjectContainer parentObject,
+							IPath fullPath,
+							int kind) {
+						return new GitModelCacheTree(parentObject, repo,
+								fullPath, fileFactory);
+					}
+				});
 	}
 
 	@Override
@@ -122,17 +92,7 @@ public class GitModelCache extends GitModelObjectContainer {
 
 	@Override
 	public GitModelObject[] getChildren() {
-		List<GitModelObject> result = new ArrayList<GitModelObject>();
-
-		for (Entry<String, Change> cacheEntry : cache.entrySet()) {
-			GitModelObject entry = extractFromCache(cacheEntry.getValue(), cacheEntry.getKey());
-			if (entry == null)
-				continue;
-
-			result.add(entry);
-		}
-
-		return result.toArray(new GitModelObject[result.size()]);
+		return children;
 	}
 
 	@Override
@@ -176,34 +136,10 @@ public class GitModelCache extends GitModelObjectContainer {
 
 	@Override
 	public void dispose() {
-		for (GitModelTree modelTree : cacheTreeMap.values())
-			modelTree.dispose();
-
-		cache.clear();
-		cacheTreeMap.clear();
-	}
-
-	private GitModelObject extractFromCache(Change change, String path) {
-		if (path.contains("/")) //$NON-NLS-1$
-			return handleCacheTree(change, path);
-
-		return fileFactory.createFileModel(this, repo, change,
-				location.append(path));
-	}
-
-	private GitModelObject handleCacheTree(Change change, String path) {
-		int firstSlash = path.indexOf("/");//$NON-NLS-1$
-		String pathKey = path.substring(0, firstSlash);
-		GitModelCacheTree cacheTree = cacheTreeMap.get(pathKey);
-		if (cacheTree == null) {
-			IPath newPath = location.append(pathKey);
-			cacheTree = new GitModelCacheTree(this, repo, newPath, fileFactory);
-			cacheTreeMap.put(pathKey, cacheTree);
+		if (children != null) {
+			for (GitModelObject object : children)
+				object.dispose();
+			children = null;
 		}
-
-		cacheTree.addChild(change, path.substring(firstSlash + 1));
-
-		return cacheTree;
 	}
-
 }
