@@ -1,6 +1,5 @@
 /*******************************************************************************
-
- * Copyright (C) 2010, Dariusz Luksza <dariusz@luksza.org>
+ * Copyright (C) 2010, 2013 Dariusz Luksza <dariusz@luksza.org> and others.
  *
  * All rights reserved. This program and the accompanying materials
  * are made available under the terms of the Eclipse Public License v1.0
@@ -9,9 +8,7 @@
  *******************************************************************************/
 package org.eclipse.egit.ui.internal.synchronize.model;
 
-import java.util.Collection;
-import java.util.HashMap;
-import java.util.Map;
+import java.util.List;
 
 import org.eclipse.core.runtime.IPath;
 
@@ -27,7 +24,7 @@ public class GitModelTree extends GitModelObjectContainer {
 	 */
 	protected final IPath path;
 
-	final Map<String, GitModelObject> cachedTreeMap = new HashMap<String, GitModelObject>();
+	private GitModelObject[] children;
 
 	/**
 	 * @param parent
@@ -66,9 +63,7 @@ public class GitModelTree extends GitModelObjectContainer {
 
 	@Override
 	public GitModelObject[] getChildren() {
-		Collection<GitModelObject> values = cachedTreeMap.values();
-
-		return values.toArray(new GitModelObject[values.size()]);
+		return children;
 	}
 
 	@Override
@@ -78,10 +73,11 @@ public class GitModelTree extends GitModelObjectContainer {
 
 	@Override
 	public void dispose() {
-		for (GitModelObject value : cachedTreeMap.values())
-			value.dispose();
-
-		cachedTreeMap.clear();
+		if (children != null) {
+			for (GitModelObject object : children)
+				object.dispose();
+			children = null;
+		}
 	}
 
 	@Override
@@ -120,4 +116,8 @@ public class GitModelTree extends GitModelObjectContainer {
 		return "ModelTree[location=" + getLocation() + "]"; //$NON-NLS-1$ //$NON-NLS-2$
 	}
 
+	void setChildren(List<GitModelObject> children) {
+		this.children = children.toArray(new GitModelObject[children.size()]);
+	}
+
 }
